@@ -134,17 +134,17 @@ void FileTreeModelTest::cleanup()
 
 void FileTreeModelTest::basic()
 {
-    QScopedPointer<DummyDocument> d1(new DummyDocument());
-    QScopedPointer<DummyDocument> d2(new DummyDocument());
+    DummyDocument d1;
+    DummyDocument d2;
 
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     // 1 because, there is always a "open widgets" node
     QCOMPARE(m.rowCount(QModelIndex()), 1);
 
-    m.documentOpened(d1.data());
+    m.documentOpened(&d1);
     QCOMPARE(m.rowCount(QModelIndex()), 2);
 
-    m.documentOpened(d2.data());
+    m.documentOpened(&d2);
     QCOMPARE(m.rowCount(QModelIndex()), 3);
 }
 
@@ -236,7 +236,7 @@ void FileTreeModelTest::buildTree_data()
 
 void FileTreeModelTest::buildTree()
 {
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     QAbstractItemModelTester tester(&m, this);
     QFETCH(const QList<DummyDocument *>, documents);
     QFETCH(ResultNode, nodes);
@@ -260,7 +260,7 @@ void FileTreeModelTest::buildTreeBatch_data()
 
 void FileTreeModelTest::buildTreeBatch()
 {
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     QAbstractItemModelTester tester(&m, this);
     QFETCH(const QList<DummyDocument *>, documents);
     QFETCH(ResultNode, nodes);
@@ -297,7 +297,7 @@ void FileTreeModelTest::buildTreeBatchPrefill_data()
 
 void FileTreeModelTest::buildTreeBatchPrefill()
 {
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     QAbstractItemModelTester tester(&m, this);
     QFETCH(const QList<DummyDocument *>, prefill);
     QFETCH(const QList<DummyDocument *>, documents);
@@ -401,7 +401,7 @@ void FileTreeModelTest::buildTreeFullPath_data()
 
 void FileTreeModelTest::buildTreeFullPath()
 {
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     QAbstractItemModelTester tester(&m, this);
     m.setShowFullPathOnRoots(true);
 
@@ -440,7 +440,7 @@ void FileTreeModelTest::listMode_data()
 
 void FileTreeModelTest::listMode()
 {
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     QAbstractItemModelTester tester(&m, this);
     m.setListMode(true);
 
@@ -503,7 +503,7 @@ void FileTreeModelTest::deleteDocument_data()
 
 void FileTreeModelTest::deleteDocument()
 {
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     QAbstractItemModelTester tester(&m, this);
     QFETCH(const QList<DummyDocument *>, documents);
     QFETCH(const QList<int>, remove);
@@ -542,7 +542,7 @@ void FileTreeModelTest::deleteDocumentBatch_data()
 
 void FileTreeModelTest::deleteDocumentBatch()
 {
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     QAbstractItemModelTester tester(&m, this);
     QFETCH(const QList<DummyDocument *>, documents);
     QFETCH(const QList<int>, remove);
@@ -586,7 +586,7 @@ void FileTreeModelTest::rename_data()
 
 void FileTreeModelTest::rename()
 {
-    KateFileTreeModel m(this);
+    KateFileTreeModel m(nullptr, this);
     QAbstractItemModelTester tester(&m, this);
     QFETCH(const QList<DummyDocument *>, documents);
     QFETCH(int, rename_idx);
