@@ -9,29 +9,21 @@
 #include "katefiletree.h"
 
 #include "filehistorywidget.h"
+#include "katefileactions.h"
 #include "katefiletreemodel.h"
 #include "katefiletreeproxymodel.h"
 
-#include "katefileactions.h"
-
 #include <ktexteditor/application.h>
 #include <ktexteditor/document.h>
 #include <ktexteditor/editor.h>
 
-#include <KApplicationTrader>
-#include <KIO/ApplicationLauncherJob>
 #include <KIO/CopyJob>
 #include <KIO/DeleteJob>
-#include <kio_version.h>
-#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
-#include <KIO/JobUiDelegateFactory>
-#else
-#include <KIO/JobUiDelegate>
-#endif
 #include <KIO/OpenFileManagerWindowJob>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KStandardAction>
+#include <KTextEditor/MainWindow>
 
 #include <QActionGroup>
 #include <QApplication>
@@ -99,8 +91,9 @@ private:
 
 // BEGIN KateFileTree
 
-KateFileTree::KateFileTree(QWidget *parent)
+KateFileTree::KateFileTree(KTextEditor::MainWindow *mainWindow, QWidget *parent)
     : QTreeView(parent)
+    , m_mainWindow(mainWindow)
 {
     setIndentation(12);
     setAllColumnsShowFocus(true);
@@ -470,29 +463,12 @@ void KateFileTree::contextMenuEvent(QContextMenuEvent *event)
 
 void KateFileTree::slotFixOpenWithMenu(QMenu *menu)
 {
-    menu->clear();
-
     KTextEditor::Document *doc = m_proxyModel->docFromIndex(m_indexContextMenu);
     if (!doc) {
         return;
     }
 
-    // get a list of appropriate services.
-    QMimeDatabase db;
-    QMimeType mime = db.mimeTypeForName(doc->mimeType());
-
-    const KService::List offers = KApplicationTrader::queryByMimeType(mime.name());
-    // for each one, insert a menu item...
-    for (const auto &service : offers) {
-        if (service->name() == QLatin1String("Kate")) {
-            continue;
-        }
-        QAction *a = menu->addAction(QIcon::fromTheme(service->icon()), service->name());
-        a->setData(service->entryPath());
-    }
-    // append "Other..." to call the KDE "open with" dialog.
-    QAction *other = menu->addAction(i18n("&Other..."));
-    other->setData(QString());
+    KateFileActions::prepareOpenWithMenu(doc->url(), menu);
 }
 
 void KateFileTree::slotOpenWithMenuAction(QAction *a)
@@ -502,18 +478,7 @@ void KateFileTree::slotOpenWithMenuAction(QAction *a)
         return;
     }
 
-    const QList<QUrl> list({doc->url()});
-
-    KService::Ptr app = KService::serviceByDesktopPath(a->data().toString());
-    // If app is null, ApplicationLauncherJob will invoke the open-with dialog
-    auto *job = new KIO::ApplicationLauncherJob(app);
-    job->setUrls(list);
-#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
-    job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, this));
-#else
-    job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, this));
-#endif
-    job->start();
+    KateFileActions::showOpenWithMenu(m_mainWindow->window(), doc->url(), a);
 }
 
 void KateFileTree::slotDocumentClose()
@@ -594,23 +559,23 @@ void KateFileTree::slotDocumentReload()
 void KateFileTree::slotOpenContainingFolder()
 {
     KTextEditor::Document *doc = m_proxyModel->docFromIndex(m_indexContextMenu);
-    if (doc) {
-        KIO::highlightInFileManager({doc->url()});
+
+    if (!doc) {
+        return;
     }
+
+    KateFileActions::openContainingFolder(doc);
 }
 
 void KateFileTree::slotCopyFilename()
 {
     KTextEditor::Document *doc = m_proxyModel->docFromIndex(m_indexContextMenu);
 
-    // TODO: the following code was improved in kate/katefileactions.cpp and should be reused here
-    //       (make sure that the mentioned bug 381052 does not reappear)
-
-    if (doc) {
-        const QUrl url = doc->url();
-        // ensure we prefer native separators, bug 381052
-        QApplication::clipboard()->setText(url.isLocalFile() ? QDir::toNativeSeparators(url.toLocalFile()) : url.url());
+    if (!doc) {
+        return;
     }
+
+    KateFileActions::copyFilePathToClipboard(doc);
 }
 
 void KateFileTree::slotRenameFile()
@@ -815,37 +780,7 @@ void KateFileTree::slotResetHistory()
 void KateFileTree::slotDocumentDelete()
 {
     KTextEditor::Document *doc = m_proxyModel->docFromIndex(m_indexContextMenu);
-
-    // TODO: the following code was improved in kate/katefileactions.cpp and should be reused here
-
-    if (!doc) {
-        return;
-    }
-
-    QUrl url = doc->url();
-
-    bool go = (KMessageBox::warningContinueCancel(this,
-                                                  i18n("Do you really want to delete file \"%1\" from storage?", url.toDisplayString()),
-                                                  i18n("Delete file?"),
-                                                  KStandardGuiItem::del(),
-                                                  KStandardGuiItem::cancel(),
-                                                  QStringLiteral("filetreedeletefile"))
-               == KMessageBox::Continue);
-
-    if (!go) {
-        return;
-    }
-
-    if (!closeDocs({doc})) {
-        return; // no extra message, the internals of ktexteditor should take care of that.
-    }
-
-    if (url.isValid()) {
-        KIO::DeleteJob *job = KIO::del(url);
-        if (!job->exec()) {
-            KMessageBox::error(this, i18n("File \"%1\" could not be deleted.", url.toDisplayString()));
-        }
-    }
+    KateFileActions::deleteDocumentFile(m_mainWindow->window(), doc);
 }
 
 // END KateFileTree
