@@ -71,6 +71,7 @@ KateFileTreeConfigPage::KateFileTreeConfigPage(QWidget *parent, KateFileTreePlug
     cmbSort->addItem(i18n("Opening Order"), static_cast<int>(KateFileTreeModel::OpeningOrderRole));
     cmbSort->addItem(i18n("Document Name"), static_cast<int>(Qt::DisplayRole));
     cmbSort->addItem(i18n("Url"), static_cast<int>(KateFileTreeModel::PathRole));
+    cmbSort->addItem(i18n("Custom Sorting"), static_cast<int>(CustomSorting));
 
     // view mode
     QHBoxLayout *lo3 = new QHBoxLayout;
@@ -143,7 +144,7 @@ QString KateFileTreeConfigPage::fullName() const
 
 QIcon KateFileTreeConfigPage::icon() const
 {
-    return QIcon::fromTheme(QLatin1String("document-open"));
+    return QIcon::fromTheme(QLatin1String("folder-documents-symbolic"));
 }
 
 void KateFileTreeConfigPage::apply()
