@@ -415,8 +415,9 @@ void ProxyItem::updateDocumentName()
 
 // END ProxyItem
 
-KateFileTreeModel::KateFileTreeModel(QObject *p)
+KateFileTreeModel::KateFileTreeModel(KTextEditor::MainWindow *mainWindow, QObject *p)
     : QAbstractItemModel(p)
+    , m_mainWindow(mainWindow)
     , m_root(new ProxyItemDir(QStringLiteral("m_root"), nullptr))
 {
     // setup default settings
@@ -490,7 +491,9 @@ void KateFileTreeModel::initModel()
 {
     beginInsertRows(QModelIndex(), 0, 0);
     Q_ASSERT(!m_widgetsRoot);
-    m_widgetsRoot = new ProxyItem(i18n("Open Widgets"), nullptr, ProxyItem::Flags(ProxyItem::Dir | ProxyItem::Widget));
+    m_widgetsRoot = new ProxyItem(i18nc("Open here is a description, i.e. 'list of widgets that are open' not a verb", "Open Widgets"),
+                                  nullptr,
+                                  ProxyItem::Flags(ProxyItem::Dir | ProxyItem::Widget));
     m_widgetsRoot->setFlags(ProxyItem::Flags(ProxyItem::Dir | ProxyItem::Widget));
     m_widgetsRoot->setIcon(QIcon::fromTheme(QStringLiteral("folder-windows")));
     m_root->addChild(m_widgetsRoot);
@@ -502,11 +505,12 @@ void KateFileTreeModel::initModel()
         documentOpened(doc);
     }
 
-    auto mw = KTextEditor::Editor::instance()->application()->activeMainWindow();
-    QWidgetList widgets;
-    QMetaObject::invokeMethod(mw->window(), "widgets", Q_RETURN_ARG(QWidgetList, widgets));
-    for (auto *w : std::as_const(widgets)) {
-        addWidget(w);
+    if (m_mainWindow) {
+        QWidgetList widgets;
+        QMetaObject::invokeMethod(m_mainWindow->window(), "widgets", Q_RETURN_ARG(QWidgetList, widgets));
+        for (auto *w : std::as_const(widgets)) {
+            addWidget(w);
+        }
     }
 }
 
@@ -1511,6 +1515,7 @@ void KateFileTreeModel::removeWidget(QWidget *w)
     beginRemoveRows(parentIdx, item->row(), item->row());
     m_widgetsRoot->removeChild(item);
     endRemoveRows();
+    delete item;
 }
 
 #include "katefiletreemodel.moc"
