@@ -2,8 +2,7 @@
 //
 //  SPDX-License-Identifier: LGPL-2.0-only
 
-#ifndef AppOutput_H
-#define AppOutput_H
+#pragma once
 
 #include <QString>
 #include <QWidget>
@@ -24,12 +23,10 @@ public:
     QString runningProcess();
 
 Q_SIGNALS:
-    void runningChanhged();
+    void runningChanged();
 
 private:
     struct Private;
     friend struct Private;
     std::unique_ptr<Private> d;
 };
-
-#endif
