@@ -7,8 +7,7 @@
    SPDX-License-Identifier: LGPL-2.0-only
 */
 
-#ifndef __KATE_CONSOLE_H__
-#define __KATE_CONSOLE_H__
+#pragma once
 
 #include <KTextEditor/Plugin>
 #include <ktexteditor/configpage.h>
@@ -28,6 +27,7 @@ class ReadOnlyPart;
 
 class KateConsole;
 class KateKonsolePluginView;
+class KPluginFactory;
 
 class KateKonsolePlugin : public KTextEditor::Plugin
 {
@@ -124,6 +124,13 @@ public:
 
     bool eventFilter(QObject *w, QEvent *e) override;
 
+    static KPluginFactory *pluginFactory();
+
+    bool hasKonsole() const
+    {
+        return pluginFactory() != nullptr;
+    }
+
 public Q_SLOTS:
     /**
      * pipe current document to console
@@ -195,6 +202,11 @@ protected:
     void showEvent(QShowEvent *ev) override;
 
 private:
+    /**
+     * plugin factory for the terminal
+     */
+    static inline KPluginFactory *s_pluginFactory = nullptr;
+
     /**
      * console part
      */
@@ -249,4 +261,3 @@ private Q_SLOTS:
      */
     static void slotEnableRunWarning();
 };
-#endif
