@@ -55,6 +55,11 @@ static const QString MEMBER_TARGET_SELECTION_RANGE = QStringLiteral("targetSelec
 static const QString MEMBER_PREVIOUS_RESULT_ID = QStringLiteral("previousResultId");
 static const QString MEMBER_QUERY = QStringLiteral("query");
 
+static QJsonValue encodeUrl(const QUrl url)
+{
+    return QJsonValue(QLatin1String(url.toEncoded()));
+}
+
 // message construction helpers
 static QJsonObject to_json(const LSPPosition &pos)
 {
@@ -69,7 +74,7 @@ static QJsonObject to_json(const LSPRange &range)
 static QJsonValue to_json(const LSPLocation &location)
 {
     if (location.uri.isValid()) {
-        return QJsonObject{{MEMBER_URI, location.uri.toString()}, {MEMBER_RANGE, to_json(location.range)}};
+        return QJsonObject{{MEMBER_URI, encodeUrl(location.uri)}, {MEMBER_RANGE, to_json(location.range)}};
     }
     return QJsonValue();
 }
@@ -130,7 +135,7 @@ static QJsonArray to_json(const QVector<LSPPosition> &positions)
 
 static QJsonObject versionedTextDocumentIdentifier(const QUrl &document, int version = -1)
 {
-    QJsonObject map{{MEMBER_URI, document.toString()}};
+    QJsonObject map{{MEMBER_URI, encodeUrl(document)}};
     if (version >= 0) {
         map[MEMBER_VERSION] = version;
     }
@@ -238,7 +243,7 @@ static QJsonObject applyWorkspaceEditResponse(const LSPApplyWorkspaceEditRespons
 
 static QJsonObject workspaceFolder(const LSPWorkspaceFolder &response)
 {
-    return QJsonObject{{MEMBER_URI, response.uri.toString()}, {QStringLiteral("name"), response.name}};
+    return QJsonObject{{MEMBER_URI, encodeUrl(response.uri)}, {QStringLiteral("name"), response.name}};
 }
 
 static QJsonObject changeConfigurationParams(const QJsonValue &settings)
@@ -391,6 +396,7 @@ static void from_json(LSPServerCapabilities &caps, const QJsonObject &json)
     auto workspace = json.value(QStringLiteral("workspace")).toObject();
     from_json(caps.workspaceFolders, workspace.value(QStringLiteral("workspaceFolders")));
     caps.selectionRangeProvider = toBoolOrObject(json.value(QStringLiteral("selectionRangeProvider")));
+    caps.inlayHintProvider = toBoolOrObject(json.value(QStringLiteral("inlayHintProvider")));
 }
 
 // follow suit; as performed in kate docmanager
@@ -672,12 +678,6 @@ static QList<LSPCompletionItem> parseDocumentCompletion(const QJsonValue &result
         items = result.toObject().value(QStringLiteral("items")).toArray();
     }
 
-    //     auto parseTextEdit = [](const QJsonObject &obj) -> LSPTextEdit {
-    //         auto newText = obj.value(QStringLiteral("newText")).toString();
-    //         auto range = parseRange(obj.value(QStringLiteral("range")).toObject());
-    //         return LSPTextEdit{range, newText};
-    //     };
-
     for (const auto &vitem : qAsConst(items)) {
         const auto &item = vitem.toObject();
         auto label = item.value(MEMBER_LABEL).toString();
@@ -691,22 +691,35 @@ static QList<LSPCompletionItem> parseDocumentCompletion(const QJsonValue &result
         if (insertText.isEmpty()) {
             insertText = label;
         }
+        LSPTextEdit lspTextEdit;
         const auto &textEdit = item.value(QStringLiteral("textEdit")).toObject();
         if (!textEdit.empty()) {
             // Not a proper implementation of textEdit, but a workaround for KDE bug #445085
             auto newText = textEdit.value(QStringLiteral("newText")).toString();
             insertText = newText;
+            lspTextEdit.newText = newText;
+            lspTextEdit.range = parseRange(textEdit.value(QStringLiteral("range")).toObject());
         }
         auto kind = static_cast<LSPCompletionItemKind>(item.value(MEMBER_KIND).toInt());
-        //         auto textEdit = parseTextEdit(item.value(QStringLiteral("textEdit")).toObject());
-
         const auto additionalTextEdits = parseTextEdit(item.value(QStringLiteral("additionalTextEdits")));
 
-        ret.push_back({label, kind, detail, doc, sortText, insertText, additionalTextEdits /*, textEdit*/});
+        QJsonValue data = item.value(QStringLiteral("data"));
+
+        ret.push_back({label, label, kind, detail, doc, sortText, insertText, additionalTextEdits, lspTextEdit, data});
     }
     return ret;
 }
 
+static LSPCompletionItem parseDocumentCompletionResolve(const QJsonValue &result)
+{
+    const auto &item = result.toObject();
+    // we only support additionalTextEdits in resolve atm
+    const auto additionalTextEdits = parseTextEdit(item.value(QStringLiteral("additionalTextEdits")));
+    LSPCompletionItem ret;
+    ret.additionalTextEdits = additionalTextEdits;
+    return ret;
+}
+
 static LSPSignatureInformation parseSignatureInformation(const QJsonObject &json)
 {
     LSPSignatureInformation info;
@@ -808,9 +821,9 @@ static LSPCommand parseCommand(const QJsonObject &result)
     return {title, command, args};
 }
 
-static QList<LSPDiagnostic> parseDiagnostics(const QJsonArray &result)
+static QVector<LSPDiagnostic> parseDiagnostics(const QJsonArray &result)
 {
-    QList<LSPDiagnostic> ret;
+    QVector<LSPDiagnostic> ret;
     for (const auto &vdiag : result) {
         auto diag = vdiag.toObject();
         auto range = parseRange(diag.value(MEMBER_RANGE).toObject());
@@ -905,6 +918,36 @@ static LSPSemanticTokensDelta parseSemanticTokensDelta(const QJsonValue &result)
     return ret;
 }
 
+static QVector<LSPInlayHint> parseInlayHints(const QJsonValue &result)
+{
+    const auto hints = result.toArray();
+    QVector<LSPInlayHint> ret;
+    for (const auto &hint : hints) {
+        LSPInlayHint h;
+        const auto &label = hint[QStringLiteral("label")];
+        if (label.isObject()) {
+            h.label = label.toObject().value(QStringLiteral("value")).toString();
+        } else {
+            h.label = label.toString();
+        }
+        h.position = parsePosition(hint[QStringLiteral("position")].toObject());
+        h.paddingLeft = hint[QStringLiteral("paddingLeft")].toBool();
+        h.paddingRight = hint[QStringLiteral("paddingRight")].toBool();
+        ret.push_back(h);
+    }
+    auto comp = [](const LSPInlayHint &l, const LSPInlayHint &r) {
+        return l.position < r.position;
+    };
+
+    // it is likely to be already sorted
+    if (!std::is_sorted(ret.begin(), ret.end(), comp)) {
+        std::sort(ret.begin(), ret.end(), comp);
+    }
+
+    // printf("%s\n", QJsonDocument(result.toArray()).toJson().constData());
+    return ret;
+}
+
 static LSPPublishDiagnosticsParams parseDiagnostics(const QJsonObject &result)
 {
     LSPPublishDiagnosticsParams ret;
@@ -1017,8 +1060,8 @@ class LSPClientServer::LSPClientServerPrivate
     QString m_langId;
     // user provided init
     QJsonValue m_init;
-    // optional workspace folders
-    FoldersType m_folders;
+    // additional tweaks
+    ExtraServerConfig m_config;
     // server process
     QProcess m_sproc;
     // server declared capabilities
@@ -1045,13 +1088,13 @@ public:
                            const QUrl &root,
                            const QString &langId,
                            const QJsonValue &init,
-                           const FoldersType &folders)
+                           ExtraServerConfig config)
         : q(_q)
         , m_server(server)
         , m_root(root)
         , m_langId(langId)
         , m_init(init)
-        , m_folders(folders)
+        , m_config(config)
     {
         // setup async reading
         QObject::connect(&m_sproc, &QProcess::readyReadStandardOutput, utils::mem_fun(&self_type::readStandardOutput, this));
@@ -1318,10 +1361,22 @@ private:
         }
     }
 
+    void applyTriggerOverride(QVector<QChar> &characters, const TriggerCharactersOverride &adjust)
+    {
+        // these are expected 'small' sets, so the simple way should do
+        for (const auto &c : adjust.exclude) {
+            characters.removeAll(c);
+        }
+        characters.append(adjust.include);
+    }
+
     void onInitializeReply(const QJsonValue &value)
     {
         // only parse parts that we use later on
         from_json(m_capabilities, value.toObject().value(QStringLiteral("capabilities")).toObject());
+        // tweak triggers as specified
+        applyTriggerOverride(m_capabilities.completionProvider.triggerCharacters, m_config.completion);
+        applyTriggerOverride(m_capabilities.signatureHelpProvider.triggerCharacters, m_config.signature);
         // finish init
         initialized();
     }
@@ -1364,6 +1419,17 @@ private:
                                                     QStringLiteral("markdown"),
                                                     QStringLiteral("plaintext")
                                                 }}
+                                            }},
+                                            {QStringLiteral("completion"), QJsonObject{
+                                                {QStringLiteral("completionItem"), QJsonObject{
+                                                    {QStringLiteral("snippetSupport"), m_config.caps.snippetSupport},
+                                                    {QStringLiteral("resolveSupport"), QJsonObject{
+                                                        {QStringLiteral("properties"), QJsonArray{ QStringLiteral("additionalTextEdits") }}
+                                                    }}
+                                                }}
+                                            }},
+                                            {QStringLiteral("inlayHint"), QJsonObject{
+                                                {QStringLiteral("dynamicRegistration"), false}
                                             }}
                                         },
                                   },
@@ -1374,7 +1440,8 @@ private:
                                   }
                                 };
         // only declare workspace support if folders so specified
-        if (m_folders) {
+        const auto &folders = m_config.folders;
+        if (folders) {
             capabilities[QStringLiteral("workspace")] = QJsonObject{{QStringLiteral("workspaceFolders"), true}};
         }
         // NOTE a typical server does not use root all that much,
@@ -1385,8 +1452,8 @@ private:
                            {QStringLiteral("capabilities"), capabilities},
                            {QStringLiteral("initializationOptions"), m_init}};
         // only add new style workspaces init if so specified
-        if (m_folders) {
-            params[QStringLiteral("workspaceFolders")] = to_json(*m_folders);
+        if (folders) {
+            params[QStringLiteral("workspaceFolders")] = to_json(*folders);
         }
         //
         write(init_request(QStringLiteral("initialize"), params), utils::mem_fun(&self_type::onInitializeReply, this));
@@ -1494,6 +1561,19 @@ public:
         return send(init_request(QStringLiteral("textDocument/completion"), params), h);
     }
 
+    RequestHandle documentCompletionResolve(const LSPCompletionItem &c, const GenericReplyHandler &h)
+    {
+        QJsonObject params;
+        params[QStringLiteral("data")] = c.data;
+        params[MEMBER_DETAIL] = c.detail;
+        params[QStringLiteral("insertText")] = c.insertText;
+        params[QStringLiteral("sortText")] = c.sortText;
+        params[QStringLiteral("textEdit")] = QJsonObject{{QStringLiteral("newText"), c.textEdit.newText}, {QStringLiteral("range"), to_json(c.textEdit.range)}};
+        params[MEMBER_LABEL] = c.originalLabel;
+        params[MEMBER_KIND] = (int)c.kind;
+        return send(init_request(QStringLiteral("completionItem/resolve"), params), h);
+    }
+
     RequestHandle signatureHelp(const QUrl &document, const LSPPosition &pos, const GenericReplyHandler &h)
     {
         auto params = textDocumentPositionParams(document, pos);
@@ -1508,7 +1588,7 @@ public:
 
     RequestHandle clangdSwitchSourceHeader(const QUrl &document, const GenericReplyHandler &h)
     {
-        auto params = QJsonObject{{MEMBER_URI, document.toString()}};
+        auto params = QJsonObject{{MEMBER_URI, encodeUrl(document)}};
         return send(init_request(QStringLiteral("textDocument/switchSourceHeader"), params), h);
     }
 
@@ -1572,6 +1652,13 @@ public:
         return send(init_request(QStringLiteral("textDocument/semanticTokens/full"), params), h);
     }
 
+    RequestHandle documentInlayHint(const QUrl &document, const LSPRange &range, const GenericReplyHandler &h)
+    {
+        auto params = textDocumentParams(document);
+        params[MEMBER_RANGE] = to_json(range);
+        return send(init_request(QStringLiteral("textDocument/inlayHint"), params), h);
+    }
+
     void executeCommand(const QString &command, const QJsonValue &args)
     {
         auto params = executeCommandParams(command, args);
@@ -1726,8 +1813,8 @@ make_handler(const ReplyHandler<ReplyType> &h, const QObject *context, typename
     };
 }
 
-LSPClientServer::LSPClientServer(const QStringList &server, const QUrl &root, const QString &langId, const QJsonValue &init, const FoldersType &folders)
-    : d(new LSPClientServerPrivate(this, server, root, langId, init, folders))
+LSPClientServer::LSPClientServer(const QStringList &server, const QUrl &root, const QString &langId, const QJsonValue &init, ExtraServerConfig config)
+    : d(new LSPClientServerPrivate(this, server, root, langId, init, config))
 {
 }
 
@@ -1830,6 +1917,12 @@ LSPClientServer::documentCompletion(const QUrl &document, const LSPPosition &pos
     return d->documentCompletion(document, pos, make_handler(h, context, parseDocumentCompletion));
 }
 
+LSPClientServer::RequestHandle
+LSPClientServer::documentCompletionResolve(const LSPCompletionItem &c, const QObject *context, const DocumentCompletionResolveReplyHandler &h)
+{
+    return d->documentCompletionResolve(c, make_handler(h, context, parseDocumentCompletionResolve));
+}
+
 LSPClientServer::RequestHandle
 LSPClientServer::signatureHelp(const QUrl &document, const LSPPosition &pos, const QObject *context, const SignatureHelpReplyHandler &h)
 {
@@ -1927,6 +2020,12 @@ LSPClientServer::documentSemanticTokensRange(const QUrl &document, const LSPRang
     return d->documentSemanticTokensFull(document, /* delta = */ false, QString(), range, make_handler(h, context, parseSemanticTokensDelta));
 }
 
+LSPClientServer::RequestHandle
+LSPClientServer::documentInlayHint(const QUrl &document, const LSPRange &range, const QObject *context, const InlayHintsReplyHandler &h)
+{
+    return d->documentInlayHint(document, range, make_handler(h, context, parseInlayHints));
+}
+
 void LSPClientServer::executeCommand(const QString &command, const QJsonValue &args)
 {
     return d->executeCommand(command, args);
