@@ -99,7 +99,22 @@ static QStringList indicationDataToStringList(const QJsonValue &indicationData)
     }
 
     return {};
-};
+}
+
+static LSPClientServer::TriggerCharactersOverride parseTriggerOverride(const QJsonValue &json)
+{
+    LSPClientServer::TriggerCharactersOverride adjust;
+    if (json.isObject()) {
+        auto ob = json.toObject();
+        for (const auto &c : ob.value(QStringLiteral("exclude")).toString()) {
+            adjust.exclude.push_back(c);
+        }
+        for (const auto &c : ob.value(QStringLiteral("include")).toString()) {
+            adjust.include.push_back(c);
+        }
+    }
+    return adjust;
+}
 
 #include <memory>
 
@@ -200,7 +215,7 @@ class LSPClientServerManagerImpl : public LSPClientServerManager
     typedef LSPClientServerManagerImpl self_type;
 
     struct ServerInfo {
-        QSharedPointer<LSPClientServer> server;
+        std::shared_ptr<LSPClientServer> server;
         // config specified server url
         QString url;
         QTime started;
@@ -212,7 +227,7 @@ class LSPClientServerManagerImpl : public LSPClientServerManager
     };
 
     struct DocumentInfo {
-        QSharedPointer<LSPClientServer> server;
+        std::shared_ptr<LSPClientServer> server;
         // merged server config as obtain from various sources
         QJsonObject config;
         KTextEditor::MovingInterface *movingInterface;
@@ -232,6 +247,7 @@ class LSPClientServerManagerImpl : public LSPClientServerManager
     QMap<QUrl, QMap<QString, ServerInfo>> m_servers;
     QHash<KTextEditor::Document *, DocumentInfo> m_docs;
     bool m_incrementalSync = false;
+    LSPClientCapabilities m_clientCapabilities;
 
     // highlightingModeRegex => language id
     std::vector<std::pair<QRegularExpression, QString>> m_highlightingModeRegexToLanguageId;
@@ -241,7 +257,7 @@ class LSPClientServerManagerImpl : public LSPClientServerManager
     // most either do not care about the id, or can find out themselves
     // (and might get confused if we pass a not so accurate one)
     QHash<QString, bool> m_documentLanguageId;
-    typedef QVector<QSharedPointer<LSPClientServer>> ServerList;
+    typedef QVector<std::shared_ptr<LSPClientServer>> ServerList;
 
     // Servers which were not found to be installed. We use this
     // variable to avoid warning more than once
@@ -299,7 +315,7 @@ public:
                 if (!s) {
                     continue;
                 }
-                disconnect(s.data(), nullptr, this, nullptr);
+                disconnect(s.get(), nullptr, this, nullptr);
                 if (s->state() != LSPClientServer::State::None) {
                     ++count;
                     s->stop(-1, -1);
@@ -333,7 +349,7 @@ public:
     }
 
     // map (highlight)mode to lsp languageId
-    QString languageId(const QString &mode)
+    QString _languageId(const QString &mode)
     {
         // query cache first
         const auto cacheIt = m_highlightingModeToLanguageIdCache.find(mode);
@@ -354,14 +370,24 @@ public:
         return QString();
     }
 
+    QString languageId(KTextEditor::Document *doc)
+    {
+        if (!doc) {
+            return {};
+        }
+
+        auto langId = _languageId(doc->highlightingMode());
+        return langId.isEmpty() ? _languageId(doc->mode()) : langId;
+    }
+
     QObject *projectPluginView(KTextEditor::MainWindow *mainWindow)
     {
         return mainWindow->pluginView(PROJECT_PLUGIN);
     }
 
-    QString documentLanguageId(const QString mode)
+    QString documentLanguageId(KTextEditor::Document *doc)
     {
-        auto langId = languageId(mode);
+        auto langId = languageId(doc);
         const auto it = m_documentLanguageId.find(langId);
         // FIXME ?? perhaps use default false
         // most servers can find out much better on their own
@@ -379,7 +405,12 @@ public:
         m_incrementalSync = inc;
     }
 
-    QSharedPointer<LSPClientServer> findServer(KTextEditor::View *view, bool updatedoc = true) override
+    LSPClientCapabilities &clientCapabilities() override
+    {
+        return m_clientCapabilities;
+    }
+
+    std::shared_ptr<LSPClientServer> findServer(KTextEditor::View *view, bool updatedoc = true) override
     {
         if (!view) {
             return nullptr;
@@ -400,7 +431,7 @@ public:
         }
 
         if (server && updatedoc) {
-            update(server.data(), false);
+            update(server.get(), false);
         }
         return server;
     }
@@ -420,7 +451,7 @@ public:
         // find entry for server(s) and move out
         for (auto &m : m_servers) {
             for (auto it = m.begin(); it != m.end();) {
-                if (!server || it->server.data() == server) {
+                if (!server || it->server.get() == server) {
                     servers.push_back(it->server);
                     it = m.erase(it);
                 } else {
@@ -441,7 +472,7 @@ public:
     {
         auto result = new LSPClientRevisionSnapshotImpl;
         for (auto it = m_docs.begin(); it != m_docs.end(); ++it) {
-            if (it->server == server) {
+            if (it->server.get() == server) {
                 // sync server to latest revision that will be recorded
                 update(it.key(), false);
                 result->add(it.key());
@@ -466,7 +497,7 @@ private:
                 continue;
             }
             // controlling server here, so disable usual state tracking response
-            disconnect(server.data(), nullptr, this, nullptr);
+            disconnect(server.get(), nullptr, this, nullptr);
             for (auto it = m_docs.begin(); it != m_docs.end();) {
                 auto &item = it.value();
                 if (item.server == server) {
@@ -523,7 +554,7 @@ private:
             ServerInfo *info = nullptr;
             for (auto &m : m_servers) {
                 for (auto &si : m) {
-                    if (si.server.data() == server) {
+                    if (si.server.get() == server) {
                         info = &si;
                         break;
                     }
@@ -552,12 +583,12 @@ private:
             // find server info to see how bad this is
             // if this is an occasional termination/crash ... ok then
             // if this happens quickly (bad/missing server, wrong cmdline/config), then no restart
-            QSharedPointer<LSPClientServer> sserver;
+            std::shared_ptr<LSPClientServer> sserver;
             QString url;
             bool retry = true;
             for (auto &m : m_servers) {
                 for (auto &si : m) {
-                    if (si.server.data() == server) {
+                    if (si.server.get() == server) {
                         url = si.url;
                         if (si.started.secsTo(QTime::currentTime()) < 60) {
                             ++si.failcount;
@@ -584,10 +615,10 @@ private:
         }
     }
 
-    QSharedPointer<LSPClientServer> _findServer(KTextEditor::View *view, KTextEditor::Document *document, QJsonObject &mergedConfig)
+    std::shared_ptr<LSPClientServer> _findServer(KTextEditor::View *view, KTextEditor::Document *document, QJsonObject &mergedConfig)
     {
         // compute the LSP standardized language id, none found => no change
-        auto langId = languageId(document->highlightingMode());
+        auto langId = languageId(document);
         if (langId.isEmpty()) {
             return nullptr;
         }
@@ -799,8 +830,18 @@ private:
             if (useWorkspace) {
                 folders = QList<LSPWorkspaceFolder>();
             }
-            server.reset(new LSPClientServer(cmdline, root, realLangId, serverConfig.value(QStringLiteral("initializationOptions")), folders));
-            connect(server.data(), &LSPClientServer::stateChanged, this, &self_type::onStateChanged, Qt::UniqueConnection);
+            // spin up using currently configured client capabilities
+            auto &caps = m_clientCapabilities;
+            // extract some more additional config
+            auto completionOverride = parseTriggerOverride(serverConfig.value(QStringLiteral("completionTriggerCharacters")));
+            auto signatureOverride = parseTriggerOverride(serverConfig.value(QStringLiteral("signatureTriggerCharacters")));
+            // request server and setup
+            server.reset(new LSPClientServer(cmdline,
+                                             root,
+                                             realLangId,
+                                             serverConfig.value(QStringLiteral("initializationOptions")),
+                                             {folders, caps, completionOverride, signatureOverride}));
+            connect(server.get(), &LSPClientServer::stateChanged, this, &self_type::onStateChanged, Qt::UniqueConnection);
             if (!server->start(m_plugin->m_debugMode)) {
                 QString message = i18n("Failed to start server: %1", cmdline.join(QLatin1Char(' ')));
                 const auto url = serverConfig.value(QStringLiteral("url")).toString();
@@ -810,12 +851,12 @@ private:
                 }
                 showMessage(message, KTextEditor::Message::Warning);
             } else {
-                showMessage(i18n("Started server %2: %1", cmdline.join(QLatin1Char(' ')), serverDescription(server.data())), KTextEditor::Message::Positive);
+                showMessage(i18n("Started server %2: %1", cmdline.join(QLatin1Char(' ')), serverDescription(server.get())), KTextEditor::Message::Positive);
                 using namespace std::placeholders;
-                server->connect(server.data(), &LSPClientServer::logMessage, this, std::bind(&self_type::onMessage, this, true, _1));
-                server->connect(server.data(), &LSPClientServer::showMessage, this, std::bind(&self_type::onMessage, this, false, _1));
-                server->connect(server.data(), &LSPClientServer::workDoneProgress, this, &self_type::onWorkDoneProgress);
-                server->connect(server.data(), &LSPClientServer::workspaceFolders, this, &self_type::onWorkspaceFolders, Qt::UniqueConnection);
+                server->connect(server.get(), &LSPClientServer::logMessage, this, std::bind(&self_type::onMessage, this, true, _1));
+                server->connect(server.get(), &LSPClientServer::showMessage, this, std::bind(&self_type::onMessage, this, false, _1));
+                server->connect(server.get(), &LSPClientServer::workDoneProgress, this, &self_type::onWorkDoneProgress);
+                server->connect(server.get(), &LSPClientServer::workspaceFolders, this, &self_type::onWorkspaceFolders, Qt::UniqueConnection);
             }
         }
         // set out param value
@@ -881,7 +922,7 @@ private:
         Q_EMIT serverChanged();
     }
 
-    void trackDocument(KTextEditor::Document *doc, const QSharedPointer<LSPClientServer> &server, QJsonObject serverConfig)
+    void trackDocument(KTextEditor::Document *doc, const std::shared_ptr<LSPClientServer> &server, QJsonObject serverConfig)
     {
         auto it = m_docs.find(doc);
         if (it == m_docs.end()) {
@@ -953,7 +994,7 @@ private:
                     (it->server)->didChange(it->url, it->version, (it->changes.empty()) ? doc->text() : QString(), it->changes);
                 }
             } else {
-                (it->server)->didOpen(it->url, it->version, documentLanguageId(doc->highlightingMode()), doc->text());
+                (it->server)->didOpen(it->url, it->version, documentLanguageId(doc), doc->text());
                 it->open = true;
             }
             it->modified = false;
@@ -969,7 +1010,7 @@ private:
     void update(LSPClientServer *server, bool force)
     {
         for (auto it = m_docs.begin(); it != m_docs.end(); ++it) {
-            if (it->server == server) {
+            if (it->server.get() == server) {
                 update(it, force);
             }
         }
@@ -1153,9 +1194,9 @@ private:
     }
 };
 
-QSharedPointer<LSPClientServerManager> LSPClientServerManager::new_(LSPClientPlugin *plugin)
+std::shared_ptr<LSPClientServerManager> LSPClientServerManager::new_(LSPClientPlugin *plugin)
 {
-    return QSharedPointer<LSPClientServerManager>(new LSPClientServerManagerImpl(plugin));
+    return std::shared_ptr<LSPClientServerManager>(new LSPClientServerManagerImpl(plugin));
 }
 
 #include "lspclientservermanager.moc"
