@@ -181,12 +181,12 @@ class LSPClientSymbolViewImpl : public QObject, public LSPClientSymbolView
 
     LSPClientPlugin *m_plugin;
     KTextEditor::MainWindow *m_mainWindow;
-    QSharedPointer<LSPClientServerManager> m_serverManager;
-    QScopedPointer<QWidget> m_toolview;
+    std::shared_ptr<LSPClientServerManager> m_serverManager;
+    std::unique_ptr<QWidget> m_toolview;
     // parent ownership
     QPointer<QTreeView> m_symbols;
     QPointer<KLineEdit> m_filter;
-    QScopedPointer<QMenu> m_popup;
+    std::unique_ptr<QMenu> m_popup;
     // initialized/updated from plugin settings
     // managed by context menu later on
     // parent ownership
@@ -195,7 +195,7 @@ class LSPClientSymbolViewImpl : public QObject, public LSPClientSymbolView
     QAction *m_treeOn;
     QAction *m_sortOn;
     // view tracking
-    QScopedPointer<LSPClientViewTracker> m_viewTracker;
+    std::unique_ptr<LSPClientViewTracker> m_viewTracker;
     // outstanding request
     LSPClientServer::RequestHandle m_handle;
     // magic request tracking cookie
@@ -224,7 +224,7 @@ class LSPClientSymbolViewImpl : public QObject, public LSPClientSymbolView
     const QIcon m_icon_var = QIcon::fromTheme(QStringLiteral("code-variable"));
 
 public:
-    LSPClientSymbolViewImpl(LSPClientPlugin *plugin, KTextEditor::MainWindow *mainWin, QSharedPointer<LSPClientServerManager> manager)
+    LSPClientSymbolViewImpl(LSPClientPlugin *plugin, KTextEditor::MainWindow *mainWin, std::shared_ptr<LSPClientServerManager> manager)
         : m_plugin(plugin)
         , m_mainWindow(mainWin)
         , m_serverManager(std::move(manager))
@@ -237,7 +237,7 @@ public:
                                                       QIcon::fromTheme(QStringLiteral("quickopen-class")),
                                                       i18n("Symbol Outline")));
 
-        m_symbols = new QTreeView(m_toolview.data());
+        m_symbols = new QTreeView(m_toolview.get());
         m_symbols->setFocusPolicy(Qt::NoFocus);
         m_symbols->setLayoutDirection(Qt::LeftToRight);
         m_toolview->layout()->setContentsMargins(0, 0, 0, 0);
@@ -245,7 +245,7 @@ public:
         m_toolview->layout()->setSpacing(0);
 
         // setup filter line edit
-        m_filter = new KLineEdit(m_toolview.data());
+        m_filter = new KLineEdit(m_toolview.get());
         m_toolview->layout()->addWidget(m_filter);
         m_filter->setPlaceholderText(i18n("Filter..."));
         m_filter->setClearButtonEnabled(true);
@@ -290,8 +290,8 @@ public:
 
         // get updated
         m_viewTracker.reset(LSPClientViewTracker::new_(plugin, mainWin, 500, 100));
-        connect(m_viewTracker.data(), &LSPClientViewTracker::newState, this, &self_type::onViewState);
-        connect(m_serverManager.data(), &LSPClientServerManager::serverChanged, this, [this]() {
+        connect(m_viewTracker.get(), &LSPClientViewTracker::newState, this, &self_type::onViewState);
+        connect(m_serverManager.get(), &LSPClientServerManager::serverChanged, this, [this]() {
             refresh(false, false);
         });
 
@@ -646,7 +646,7 @@ private Q_SLOTS:
     }
 };
 
-LSPClientSymbolView *LSPClientSymbolView::new_(LSPClientPlugin *plugin, KTextEditor::MainWindow *mainWin, QSharedPointer<LSPClientServerManager> manager)
+LSPClientSymbolView *LSPClientSymbolView::new_(LSPClientPlugin *plugin, KTextEditor::MainWindow *mainWin, std::shared_ptr<LSPClientServerManager> manager)
 {
     return new LSPClientSymbolViewImpl(plugin, mainWin, std::move(manager));
 }
