@@ -11,7 +11,9 @@
 #include <KTextEditor/MovingInterface>
 #include <KTextEditor/View>
 
-SemanticHighlighter::SemanticHighlighter(QSharedPointer<LSPClientServerManager> serverManager, QObject *parent)
+#include <ktexteditor_utils.h>
+
+SemanticHighlighter::SemanticHighlighter(std::shared_ptr<LSPClientServerManager> serverManager, QObject *parent)
     : QObject(parent)
     , m_serverManager(std::move(serverManager))
 {
@@ -21,17 +23,6 @@ SemanticHighlighter::SemanticHighlighter(QSharedPointer<LSPClientServerManager>
     });
 }
 
-static KTextEditor::Range getCurrentViewLinesRange(KTextEditor::View *view)
-{
-    Q_ASSERT(view);
-
-    auto doc = view->document();
-    auto first = view->firstDisplayedLine();
-    auto last = view->lastDisplayedLine();
-    auto lastLineLen = doc->line(last).size();
-    return KTextEditor::Range(first, 0, last, lastLineLen);
-}
-
 void SemanticHighlighter::doSemanticHighlighting(KTextEditor::View *view, bool textChanged)
 {
     // start the timer
@@ -90,7 +81,7 @@ void SemanticHighlighter::doSemanticHighlighting_impl(KTextEditor::View *view)
     };
 
     if (caps.semanticTokenProvider.range) {
-        server->documentSemanticTokensRange(doc->url(), getCurrentViewLinesRange(view), this, h);
+        server->documentSemanticTokensRange(doc->url(), Utils::getVisibleRange(view), this, h);
     } else if (caps.semanticTokenProvider.fullDelta) {
         auto prevResultId = previousResultIdForDoc(doc);
         server->documentSemanticTokensFullDelta(doc->url(), prevResultId, this, h);
