@@ -5,13 +5,14 @@
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
-#ifndef KATE_PROJECT_CODE_ANALYSIS_TOOL_H
-#define KATE_PROJECT_CODE_ANALYSIS_TOOL_H
+#pragma once
 
 #include <QObject>
 #include <QString>
 #include <QStringList>
 
+#include "diagnostics/diagnostic_types.h"
+
 class KateProject;
 namespace KTextEditor
 {
@@ -90,7 +91,7 @@ public:
      * @param line
      * @return file, line, severity, message
      */
-    virtual QStringList parseLine(const QString &line) const = 0;
+    virtual FileDiagnostics parseLine(const QString &line) const = 0;
 
     /**
      * Tells the tool runner if the returned process exit code
@@ -130,5 +131,3 @@ private:
 };
 
 Q_DECLARE_METATYPE(KateProjectCodeAnalysisTool *)
-
-#endif // KATE_PROJECT_CODE_ANALYSIS_TOOL_H
