@@ -5,21 +5,25 @@
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
-#include "hostprocess.h"
 #include "kateprojectinfoviewcodeanalysis.h"
+#include "hostprocess.h"
 #include "kateproject.h"
 #include "kateprojectcodeanalysistool.h"
 #include "kateprojectpluginview.h"
 #include "tools/codeanalysisselector.h"
 
+#include "diagnostics/diagnostic_types.h"
+#include "diagnostics/diagnosticview.h"
+#include "ktexteditor_utils.h"
+
 #include <QFileInfo>
 #include <QHBoxLayout>
+#include <QSortFilterProxyModel>
 #include <QStandardPaths>
 #include <QToolTip>
 #include <QVBoxLayout>
 
 #include <KLocalizedString>
-#include <KMessageWidget>
 #include <QTimer>
 
 #include <KTextEditor/MainWindow>
@@ -27,33 +31,14 @@
 KateProjectInfoViewCodeAnalysis::KateProjectInfoViewCodeAnalysis(KateProjectPluginView *pluginView, KateProject *project)
     : m_pluginView(pluginView)
     , m_project(project)
-    , m_messageWidget(nullptr)
     , m_startStopAnalysis(new QPushButton(i18n("Start Analysis...")))
-    , m_treeView(new QTreeView(this))
-    , m_model(new QStandardItemModel(m_treeView))
     , m_analyzer(nullptr)
     , m_analysisTool(nullptr)
     , m_toolSelector(new QComboBox())
+    , m_diagnosticProvider(new DiagnosticsProvider(this))
 {
-    /**
-     * default style
-     */
-    m_treeView->setEditTriggers(QAbstractItemView::NoEditTriggers);
-    m_treeView->setUniformRowHeights(true);
-    m_treeView->setRootIsDecorated(false);
-    m_model->setHorizontalHeaderLabels(QStringList() << i18n("File") << i18n("Line") << i18n("Severity") << i18n("Message"));
-
-    /**
-     * attach model
-     * kill selection model
-     */
-    QItemSelectionModel *m = m_treeView->selectionModel();
-    m_treeView->setModel(m_model);
-    delete m;
-
-    m_treeView->setSortingEnabled(true);
-    m_treeView->sortByColumn(1, Qt::AscendingOrder);
-    m_treeView->sortByColumn(2, Qt::AscendingOrder);
+    m_diagnosticProvider->setObjectName(QStringLiteral("CodeAnalysisDiagnosticProvider"));
+    Utils::registerDiagnosticsProvider(m_diagnosticProvider, m_pluginView->mainWindow());
 
     /**
      * Connect selection change callback
@@ -64,6 +49,7 @@ KateProjectInfoViewCodeAnalysis::KateProjectInfoViewCodeAnalysis(KateProjectPlug
             this,
             &KateProjectInfoViewCodeAnalysis::slotToolSelectionChanged);
     m_toolSelector->setModel(KateProjectCodeAnalysisSelector::model(this));
+    m_toolSelector->setSizeAdjustPolicy(QComboBox::AdjustToContents);
 
     /**
      * layout widget
@@ -85,18 +71,18 @@ KateProjectInfoViewCodeAnalysis::KateProjectInfoViewCodeAnalysis(KateProjectPlug
     hlayout->addWidget(m_startStopAnalysis);
     hlayout->addStretch();
     // below: result list...
-    layout->addWidget(m_treeView);
+    layout->addStretch();
     setLayout(layout);
 
     /**
      * connect needed signals
      */
     connect(m_startStopAnalysis, &QPushButton::clicked, this, &KateProjectInfoViewCodeAnalysis::slotStartStopClicked);
-    connect(m_treeView, &QTreeView::clicked, this, &KateProjectInfoViewCodeAnalysis::slotClicked);
 }
 
 KateProjectInfoViewCodeAnalysis::~KateProjectInfoViewCodeAnalysis()
 {
+    Utils::unregisterDiagnosticsProvider(m_diagnosticProvider, m_pluginView->mainWindow());
     if (m_analyzer && m_analyzer->state() != QProcess::NotRunning) {
         m_analyzer->kill();
         m_analyzer->blockSignals(true);
@@ -108,9 +94,11 @@ KateProjectInfoViewCodeAnalysis::~KateProjectInfoViewCodeAnalysis()
 void KateProjectInfoViewCodeAnalysis::slotToolSelectionChanged(int)
 {
     m_analysisTool = m_toolSelector->currentData(Qt::UserRole + 1).value<KateProjectCodeAnalysisTool *>();
-    m_toolInfoText = i18n("%1<br/><br/>The tool will be run on all project files which match this list of file extensions:<br/><br/><b>%2</b>",
-                          m_analysisTool->description(),
-                          m_analysisTool->fileExtensions());
+    if (m_analysisTool) {
+        m_toolInfoText = i18n("%1<br/><br/>The tool will be run on all project files which match this list of file extensions:<br/><br/><b>%2</b>",
+                              m_analysisTool->description(),
+                              m_analysisTool->fileExtensions());
+    }
 }
 
 void KateProjectInfoViewCodeAnalysis::slotStartStopClicked()
@@ -125,7 +113,7 @@ void KateProjectInfoViewCodeAnalysis::slotStartStopClicked()
     /**
      * clear existing entries
      */
-    m_model->removeRows(0, m_model->rowCount(), QModelIndex());
+    Q_EMIT m_diagnosticProvider->requestClearDiagnostics(m_diagnosticProvider);
 
     /**
      * launch selected tool
@@ -140,22 +128,12 @@ void KateProjectInfoViewCodeAnalysis::slotStartStopClicked()
     // ensure we only run the code analyzer from PATH
     const QString fullExecutable = safeExecutableName(m_analysisTool->path());
     if (!fullExecutable.isEmpty()) {
+        m_analyzer->setWorkingDirectory(m_project->baseDir());
         startHostProcess(*m_analyzer, fullExecutable, m_analysisTool->arguments());
     }
 
-    if (m_messageWidget) {
-        delete m_messageWidget;
-        m_messageWidget = nullptr;
-    }
-
     if (fullExecutable.isEmpty() || !m_analyzer->waitForStarted()) {
-        m_messageWidget = new KMessageWidget(this);
-        m_messageWidget->setCloseButtonVisible(true);
-        m_messageWidget->setMessageType(KMessageWidget::Warning);
-        m_messageWidget->setWordWrap(false);
-        m_messageWidget->setText(m_analysisTool->notInstalledMessage());
-        static_cast<QVBoxLayout *>(layout())->addWidget(m_messageWidget);
-        m_messageWidget->animatedShow();
+        Utils::showMessage(m_analysisTool->notInstalledMessage(), {}, i18n("CodeAnalysis"), MessageType::Warning);
         return;
     }
 
@@ -176,95 +154,50 @@ void KateProjectInfoViewCodeAnalysis::slotReadyRead()
     /**
      * get results of analysis
      */
+    m_errOutput = {};
+    QHash<QUrl, QVector<Diagnostic>> fileDiagnostics;
     while (m_analyzer->canReadLine()) {
         /**
          * get one line, split it, skip it, if too few elements
          */
-        QString line = QString::fromLocal8Bit(m_analyzer->readLine());
-        QStringList elements = m_analysisTool->parseLine(line);
-        if (elements.size() < 4) {
+        auto rawLine = m_analyzer->readLine();
+        QString line = QString::fromLocal8Bit(rawLine);
+        FileDiagnostics fd = m_analysisTool->parseLine(line);
+        if (!fd.uri.isValid()) {
+            m_errOutput += rawLine;
             continue;
         }
-
-        /**
-         * feed into model
-         */
-        QList<QStandardItem *> items;
-        QStandardItem *fileNameItem = new QStandardItem(QFileInfo(elements[0]).fileName());
-        fileNameItem->setToolTip(elements[0]);
-        items << fileNameItem;
-        items << new QStandardItem(elements[1]);
-        items << new QStandardItem(elements[2]);
-        const auto message = elements[3].simplified();
-        auto messageItem = new QStandardItem(message);
-        messageItem->setToolTip(message);
-        items << messageItem;
-        m_model->appendRow(items);
+        fileDiagnostics[fd.uri] << fd.diagnostics;
     }
 
-    /**
-     * tree view polish ;)
-     */
-    m_treeView->resizeColumnToContents(2);
-    m_treeView->resizeColumnToContents(1);
-    m_treeView->resizeColumnToContents(0);
-}
-
-void KateProjectInfoViewCodeAnalysis::slotClicked(const QModelIndex &index)
-{
-    /**
-     * get path
-     */
-    QString filePath = m_model->item(index.row(), 0)->toolTip();
-    if (filePath.isEmpty()) {
-        return;
+    for (auto it = fileDiagnostics.cbegin(); it != fileDiagnostics.cend(); ++it) {
+        m_diagnosticProvider->diagnosticsAdded(FileDiagnostics{it.key(), it.value()});
     }
 
-    /**
-     * create view
-     */
-    KTextEditor::View *view = m_pluginView->mainWindow()->openUrl(QUrl::fromLocalFile(filePath));
-    if (!view) {
-        return;
-    }
-
-    /**
-     * set cursor, if possible
-     */
-    int line = m_model->item(index.row(), 1)->text().toInt();
-    if (line >= 1) {
-        view->setCursorPosition(KTextEditor::Cursor(line - 1, 0));
+    if (!fileDiagnostics.isEmpty()) {
+        m_diagnosticProvider->showDiagnosticsView();
     }
 }
 
 void KateProjectInfoViewCodeAnalysis::finished(int exitCode, QProcess::ExitStatus)
 {
     m_startStopAnalysis->setEnabled(true);
-    m_messageWidget = new KMessageWidget(this);
-    m_messageWidget->setCloseButtonVisible(true);
-    m_messageWidget->setWordWrap(false);
 
     if (m_analysisTool->isSuccessfulExitCode(exitCode)) {
         // normally 0 is successful but there are exceptions
-        m_messageWidget->setMessageType(KMessageWidget::Information);
-        m_messageWidget->setText(i18np("Analysis on %1 file finished.", "Analysis on %1 files finished.", m_analysisTool->getActualFilesCount()));
-
-        // hide after 3 seconds
-        QTimer::singleShot(3000, this, [this]() {
-            if (m_messageWidget) {
-                m_messageWidget->animatedHide();
-            }
-        });
+        const QString msg = i18ncp(
+            "Message to the user that analysis finished. %1 is the name of the program that did the analysis, %2 is a number. e.g., [clang-tidy]Analysis on 5 "
+            "files finished",
+            "[%1]Analysis on %2 file finished.",
+            "[%1]Analysis on %2 files finished.",
+            m_analysisTool->name(),
+            m_analysisTool->getActualFilesCount());
+        // We only log here because once the analysis starts, the user will be taken to diagnosticview to see the results
+        Utils::showMessage(msg, {}, i18n("CodeAnalysis"), MessageType::Log, m_pluginView->mainWindow());
     } else {
-        // unfortunately, output was eaten by slotReadyRead()
-        // TODO: get stderr output, show it here
-        m_messageWidget->setMessageType(KMessageWidget::Warning);
-        m_messageWidget->setText(i18np("Analysis on %1 file failed with exit code %2.",
-                                       "Analysis on %1 files failed with exit code %2.",
-                                       m_analysisTool->getActualFilesCount(),
-                                       exitCode));
+        const QString err = QString::fromUtf8(m_errOutput);
+        const QString message = i18n("Analysis failed with exit code %1, Error: %2", exitCode, err);
+        Utils::showMessage(message, {}, i18n("CodeAnalysis"), MessageType::Error, m_pluginView->mainWindow());
     }
-
-    static_cast<QVBoxLayout *>(layout())->addWidget(m_messageWidget);
-    m_messageWidget->animatedShow();
+    m_errOutput = {};
 }
