@@ -5,8 +5,7 @@
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
-#ifndef _KATE_PROJECT_PLUGIN_H_
-#define _KATE_PROJECT_PLUGIN_H_
+#pragma once
 
 #include <unordered_map>
 
@@ -48,8 +47,7 @@ public:
     /**
      * Create new project for given project filename.
      * Null pointer if no project can be opened.
-     * File name will be canonicalized!
-     * @param fileName canonicalized file name for the project
+     * @param fileName file name for the project
      * @return project or null if not openable
      */
     KateProject *createProjectForFileName(const QString &fileName);
@@ -296,5 +294,3 @@ private:
      */
     QThreadPool m_threadPool;
 };
-
-#endif
