@@ -5,8 +5,7 @@
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
-#ifndef _KATE_PROJECT_PLUGIN_VIEW_H_
-#define _KATE_PROJECT_PLUGIN_VIEW_H_
+#pragma once
 
 #include <QComboBox>
 #include <QMenu>
@@ -134,6 +133,8 @@ public Q_SLOTS:
 
     void runCmdInTerminal(const QString &workingDir, const QString &cmd);
 
+    void updateGitBranchButton(KateProject *project);
+
 private Q_SLOTS:
     /**
      * Plugin config updated
@@ -317,9 +318,10 @@ private:
 
 private:
     /**
-     * Watches for changes to .git/indiex
+     * Watches for changes to .git/index
+     * If this is non-empty, we registered that file in the project watcher
      */
-    QFileSystemWatcher m_gitChangedWatcher;
+    QString m_gitChangedWatcherFile;
 
     /**
      * our plugin
@@ -421,24 +423,8 @@ private:
     QAction *m_projectCloseAllAction;
     QAction *m_projectCloseWithoutDocumentsAction;
 
-    class FixedView
-    {
-    public:
-        QPointer<KTextEditor::View> view;
-        QPointer<QMenu> defaultMenu;
-
-        void restoreMenu()
-        {
-            if (view && defaultMenu) {
-                view->setContextMenu(defaultMenu);
-            }
-        }
-    };
-
     /**
-     * Fixed view for viewing diffs
+      checkout branch button in the statusbar
      */
-    FixedView m_fixedView;
+    std::unique_ptr<QToolButton> m_branchBtn = nullptr;
 };
-
-#endif
