@@ -5,8 +5,7 @@
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
-#ifndef KATE_PROJECT_WORKER_H
-#define KATE_PROJECT_WORKER_H
+#pragma once
 
 #include "kateproject.h"
 
@@ -65,7 +64,7 @@ private:
     static QVector<QString> filesFromSubversion(const QDir &dir, bool recursive);
     static QVector<QString> filesFromDarcs(const QDir &dir, bool recursive);
     static QVector<QString> filesFromFossil(const QDir &dir, bool recursive);
-    static QVector<QString> filesFromDirectory(const QDir &dir, bool recursive, const QStringList &filters);
+    static QVector<QString> filesFromDirectory(QDir dir, bool recursive, bool hidden, const QStringList &filters);
 
     static QVector<QString> gitFiles(const QDir &dir, bool recursive, const QStringList &args);
 
@@ -88,5 +87,3 @@ private:
     const QVariantMap m_projectMap;
     const bool m_force;
 };
-
-#endif
