@@ -8,157 +8,27 @@
 #include "gitwidget.h"
 #include "kateprojectpluginview.h"
 
-#include <QCoreApplication>
-#include <QFileInfo>
-#include <QKeyEvent>
-#include <QLineEdit>
-#include <QPainter>
 #include <QProcess>
-#include <QSortFilterProxyModel>
-#include <QStandardItemModel>
-#include <QStyledItemDelegate>
-#include <QTemporaryFile>
-#include <QTextDocument>
 #include <QTreeView>
-#include <QVBoxLayout>
 #include <QWidget>
-#include <QtConcurrentRun>
 
-#include <KTextEditor/MainWindow>
-#include <KTextEditor/Message>
 #include <KTextEditor/View>
 
 #include <KLocalizedString>
 
 #include <drawing_utils.h>
-#include <kfts_fuzzy_match.h>
 
 #include <gitprocess.h>
 
-constexpr int StashIndexRole = Qt::UserRole + 2;
-
-class StashFilterModel final : public QSortFilterProxyModel
-{
-public:
-    StashFilterModel(QObject *parent = nullptr)
-        : QSortFilterProxyModel(parent)
-    {
-    }
-
-    Q_SLOT void setFilterString(const QString &string)
-    {
-        beginResetModel();
-        m_pattern = string;
-        endResetModel();
-    }
-
-protected:
-    bool lessThan(const QModelIndex &sourceLeft, const QModelIndex &sourceRight) const override
-    {
-        return sourceLeft.data(FuzzyScore).toInt() < sourceRight.data(FuzzyScore).toInt();
-    }
-
-    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override
-    {
-        if (m_pattern.isEmpty()) {
-            return true;
-        }
-
-        int score = 0;
-        const auto idx = sourceModel()->index(sourceRow, 0, sourceParent);
-        const QString string = idx.data().toString();
-        const bool res = kfts::fuzzy_match(m_pattern, string, score);
-        sourceModel()->setData(idx, score, FuzzyScore);
-        return res;
-    }
-
-private:
-    static constexpr int FuzzyScore = Qt::UserRole + 1;
-    QString m_pattern;
-};
-
-class StyleDelegate : public QStyledItemDelegate
-{
-public:
-    StyleDelegate(QObject *parent = nullptr)
-        : QStyledItemDelegate(parent)
-    {
-    }
-
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override
-    {
-        QStyleOptionViewItem options = option;
-        initStyleOption(&options, index);
-
-        QString name = index.data().toString();
-
-        QVector<QTextLayout::FormatRange> formats;
-
-        int colon = name.indexOf(QLatin1Char(':'));
-        QString stashMessage = name.mid(colon + 1, name.length() - (colon + 1));
-        ++colon;
-
-        QTextCharFormat bold;
-        bold.setFontWeight(QFont::Bold);
-        formats.append({0, colon, bold});
-
-        QTextCharFormat fmt;
-        fmt.setForeground(options.palette.link());
-        fmt.setFontWeight(QFont::Bold);
-        auto resFmts = kfts::get_fuzzy_match_formats(m_filterString, stashMessage, colon, fmt);
-
-        formats.append(resFmts);
-
-        painter->save();
-
-        //        // paint background
-        if (option.state & QStyle::State_Selected) {
-            painter->fillRect(option.rect, option.palette.highlight());
-        } else {
-            painter->fillRect(option.rect, option.palette.base());
-        }
-
-        options.text = QString(); // clear old text
-        options.widget->style()->drawControl(QStyle::CE_ItemViewItem, &options, painter, options.widget);
-
-        Utils::paintItemViewText(painter, name, options, formats);
-
-        painter->restore();
-    }
-
-public Q_SLOTS:
-    void setFilterString(const QString &text)
-    {
-        m_filterString = text;
-    }
-
-private:
-    QString m_filterString;
-};
-
 StashDialog::StashDialog(QWidget *parent, QWidget *window, const QString &gitPath)
-    : QuickDialog(parent, window)
+    : HUDDialog(parent, window)
     , m_gitPath(gitPath)
 {
-    m_model = new QStandardItemModel(this);
-    m_proxyModel = new StashFilterModel(this);
-    m_proxyModel->setSourceModel(m_model);
-    m_treeView.setModel(m_proxyModel);
-
-    StyleDelegate *delegate = new StyleDelegate(this);
-    m_treeView.setItemDelegateForColumn(0, delegate);
-    connect(&m_lineEdit, &QLineEdit::textChanged, this, [this, delegate](const QString &string) {
-        m_proxyModel->setFilterString(string);
-        delegate->setFilterString(string);
-        // reselect first
-        m_treeView.setCurrentIndex(m_proxyModel->index(0, 0));
-    });
-    m_proxyModel->setFilterRole(Qt::DisplayRole);
 }
 
 void StashDialog::openDialog(StashMode m)
 {
-    m_model->clear();
+    setStringList({});
 
     switch (m) {
     case StashMode::Stash:
@@ -190,7 +60,20 @@ void StashDialog::openDialog(StashMode m)
     exec();
 }
 
-void StashDialog::slotReturnPressed()
+static QString getStashIndex(const QModelIndex &index)
+{
+    QString s = index.data().toString();
+    if (s.isEmpty() || !s.startsWith(QStringLiteral("stash@{"))) {
+        return {};
+    }
+    static QRegularExpression re(QStringLiteral("stash@{(.*)}"));
+    const auto match = re.match(s);
+    if (!match.hasMatch())
+        return {};
+    return match.captured(1);
+}
+
+void StashDialog::slotReturnPressed(const QModelIndex &index)
 {
     switch (m_currentMode) {
     case StashMode::Stash:
@@ -202,17 +85,26 @@ void StashDialog::slotReturnPressed()
     case StashMode::StashUntrackIncluded:
         stash(false, true);
         break;
+    default:
+        break;
+    }
+
+    auto stashIndex = getStashIndex(index);
+    if (stashIndex.isEmpty())
+        return;
+
+    switch (m_currentMode) {
     case StashMode::StashApply:
-        applyStash(m_treeView.currentIndex().data(StashIndexRole).toByteArray());
+        applyStash(stashIndex);
         break;
     case StashMode::StashPop:
-        popStash(m_treeView.currentIndex().data(StashIndexRole).toByteArray());
+        popStash(stashIndex);
         break;
     case StashMode::StashDrop:
-        dropStash(m_treeView.currentIndex().data(StashIndexRole).toByteArray());
+        dropStash(stashIndex);
         break;
     case StashMode::ShowStashContent:
-        showStash(m_treeView.currentIndex().data(StashIndexRole).toByteArray());
+        showStash(stashIndex);
         break;
     default:
         break;
@@ -263,39 +155,22 @@ void StashDialog::getStashList()
     auto git = gitp({QStringLiteral("stash"), QStringLiteral("list")});
     startHostProcess(*git, QProcess::ReadOnly);
 
-    QList<QByteArray> stashList;
+    QStringList stashList;
     if (git->waitForStarted() && git->waitForFinished(-1)) {
         if (git->exitStatus() == QProcess::NormalExit && git->exitCode() == 0) {
-            stashList = git->readAllStandardOutput().split('\n');
+            stashList = QString::fromUtf8(git->readAllStandardOutput()).split(QLatin1Char('\n'));
+            setStringList(stashList);
         } else {
             Q_EMIT message(i18n("Failed to get stash list. Error: ") + QString::fromUtf8(git->readAll()), true);
         }
     }
-
-    // format stash@{}: message
-    for (const auto &stash : qAsConst(stashList)) {
-        if (!stash.startsWith("stash@{")) {
-            continue;
-        }
-        int brackCloseIdx = stash.indexOf('}', 7);
-
-        if (brackCloseIdx < 0) {
-            continue;
-        }
-
-        QByteArray stashIdx = stash.mid(0, brackCloseIdx + 1);
-
-        QStandardItem *item = new QStandardItem(QString::fromUtf8(stash));
-        item->setData(stashIdx, StashIndexRole);
-        m_model->appendRow(item);
-    }
 }
 
-void StashDialog::popStash(const QByteArray &index, const QString &command)
+void StashDialog::popStash(const QString &index, const QString &command)
 {
     QStringList args{QStringLiteral("stash"), command};
     if (!index.isEmpty()) {
-        args.append(QString::fromUtf8(index));
+        args.append(index);
     }
     auto git = gitp(args);
 
@@ -323,23 +198,23 @@ void StashDialog::popStash(const QByteArray &index, const QString &command)
     startHostProcess(*git, QProcess::ReadOnly);
 }
 
-void StashDialog::applyStash(const QByteArray &index)
+void StashDialog::applyStash(const QString &index)
 {
     popStash(index, QStringLiteral("apply"));
 }
 
-void StashDialog::dropStash(const QByteArray &index)
+void StashDialog::dropStash(const QString &index)
 {
     popStash(index, QStringLiteral("drop"));
 }
 
-void StashDialog::showStash(const QByteArray &index)
+void StashDialog::showStash(const QString &index)
 {
     if (index.isEmpty()) {
         return;
     }
 
-    auto git = gitp({QStringLiteral("stash"), QStringLiteral("show"), QStringLiteral("-p"), QString::fromUtf8(index)});
+    auto git = gitp({QStringLiteral("stash"), QStringLiteral("show"), QStringLiteral("-p"), index});
 
     connect(git, &QProcess::finished, this, [this, git](int exitCode, QProcess::ExitStatus es) {
         if (es != QProcess::NormalExit || exitCode != 0) {
