@@ -5,8 +5,10 @@
 */
 #include "rainbowparens_plugin.h"
 
+#include <QGradient>
 #include <QIcon>
 #include <QLabel>
+#include <QPainter>
 #include <QScopeGuard>
 #include <QTimer>
 #include <QVBoxLayout>
@@ -45,16 +47,17 @@ void RainbowParenPlugin::readConfig()
             attr = new KTextEditor::Attribute;
         }
     }
+
     KConfigGroup config(KSharedConfig::openConfig(), "ColoredBrackets");
-    QColor br = config.readEntry("color1", QStringLiteral("#FFFF00"));
+    QColor br = config.readEntry("color1", QStringLiteral("#1275ef"));
     attrs[0]->setForeground(br);
-    br = config.readEntry("color2", QStringLiteral("#FF4797"));
+    br = config.readEntry("color2", QStringLiteral("#f83c1f"));
     attrs[1]->setForeground(br);
-    br = config.readEntry("color3", QStringLiteral("#67F058"));
+    br = config.readEntry("color3", QStringLiteral("#9dba1e"));
     attrs[2]->setForeground(br);
-    br = config.readEntry("color4", QStringLiteral("#FC834A"));
+    br = config.readEntry("color4", QStringLiteral("#e219e2"));
     attrs[3]->setForeground(br);
-    br = config.readEntry("color5", QStringLiteral("#3A86FF"));
+    br = config.readEntry("color5", QStringLiteral("#37d21c"));
     attrs[4]->setForeground(br);
 }
 
@@ -424,9 +427,38 @@ QString RainbowParenConfigPage::fullName() const
     return i18n("Colored Brackets Settings");
 }
 
+static QIcon ColoredBracketsIcon(const QWidget *_this)
+{
+    const auto icon = QIcon::fromTheme(QStringLiteral("code-context"));
+    auto p = icon.pixmap(QSize(16, 16));
+    if (p.isNull()) {
+        QRect r(QPoint(0, 0), _this->devicePixelRatioF() * QSize(16, 16));
+        QPixmap pix(r.size());
+        pix.fill(Qt::transparent);
+        QPainter paint(&pix);
+        if (paint.fontMetrics().height() > r.height()) {
+            auto f = paint.font();
+            f.setPixelSize(14);
+            paint.setFont(f);
+        }
+        paint.drawText(r, Qt::AlignCenter, QStringLiteral("{.}"));
+        paint.end();
+        p = pix;
+    }
+
+    QPainter paint(&p);
+    paint.setCompositionMode(QPainter::CompositionMode_SourceIn);
+    paint.fillRect(p.rect(), QBrush(QGradient(QGradient::Preset::FruitBlend)));
+    paint.end();
+    return p;
+}
+
 QIcon RainbowParenConfigPage::icon() const
 {
-    return {};
+    if (m_icon.isNull()) {
+        const_cast<RainbowParenConfigPage *>(this)->m_icon = ColoredBracketsIcon(this);
+    }
+    return m_icon;
 }
 
 void RainbowParenConfigPage::apply()
