@@ -4,8 +4,7 @@
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#ifndef _PLUGIN_SEARCH_H_
-#define _PLUGIN_SEARCH_H_
+#pragma once
 
 #include <KTextEditor/Command>
 #include <KTextEditor/Message>
@@ -158,6 +157,9 @@ private Q_SLOTS:
     void showExportMatchesDialog();
     void customResMenuRequested(const QPoint &pos);
 
+    void cutSearchedLines();
+    void copySearchedLines();
+
 Q_SIGNALS:
     void searchBusy(bool busy);
 
@@ -165,6 +167,9 @@ protected:
     bool eventFilter(QObject *obj, QEvent *ev) override;
 
 private:
+    QVector<int> getDocumentSearchMarkedLines(KTextEditor::Document *currentDocument);
+    void setClipboardFromDocumentLines(const KTextEditor::Document *currentDocument, const QVector<int> lineNumbers);
+
     QStringList filterFiles(const QStringList &fileList) const;
     void startDiskFileSearch(const QStringList &fileList, const QRegularExpression &reg, bool includeBinaryFiles);
     void cancelDiskFileSearch();
@@ -220,6 +225,4 @@ private:
     KTextEditor::MainWindow *m_mainWindow;
 };
 
-#endif
-
 // kate: space-indent on; indent-width 4; replace-tabs on;
