@@ -7,15 +7,15 @@
  *  SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
-#ifndef __KATE_SNIPPET_GLOBAL_H__
-#define __KATE_SNIPPET_GLOBAL_H__
+#pragma once
+
+#include "snippetcompletionmodel.h"
 
 #include <QPointer>
 #include <QVariant>
 
 #include <KTextEditor/View>
 
-class SnippetCompletionModel;
 class Snippet;
 
 /**
@@ -48,7 +48,7 @@ public:
      */
     SnippetCompletionModel *completionModel()
     {
-        return m_model.data();
+        return &m_model;
     }
 
 public Q_SLOTS:
@@ -62,8 +62,6 @@ public Q_SLOTS:
 
 private:
     static KateSnippetGlobal *s_self;
-    QScopedPointer<SnippetCompletionModel> m_model;
+    SnippetCompletionModel m_model;
     QPointer<KTextEditor::View> m_activeViewForDialog;
 };
-
-#endif
