@@ -5,13 +5,14 @@
    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
-#ifndef KTEXTEDITOR_TAB_SWITCHER_FILES_MODEL_H
-#define KTEXTEDITOR_TAB_SWITCHER_FILES_MODEL_H
+#pragma once
 
 #include <QAbstractTableModel>
 #include <QIcon>
 #include <QString>
 
+#include <doc_or_widget.h>
+
 namespace KTextEditor
 {
 class Document;
@@ -25,9 +26,9 @@ namespace detail
 class FilenameListItem
 {
 public:
-    FilenameListItem(KTextEditor::Document *doc);
+    FilenameListItem(DocOrWidget doc);
 
-    KTextEditor::Document *document;
+    DocOrWidget document;
     QIcon icon() const;
     QString documentName() const;
     QString fullPath() const;
@@ -46,8 +47,8 @@ class TabswitcherFilesModel : public QAbstractTableModel
 public:
     explicit TabswitcherFilesModel(QObject *parent = nullptr);
     ~TabswitcherFilesModel() override = default;
-    bool insertDocument(int row, KTextEditor::Document *document);
-    bool removeDocument(KTextEditor::Document *document);
+    bool insertDocument(int row, DocOrWidget document);
+    bool removeDocument(DocOrWidget document);
 
     /**
      * Clears all data from the model
@@ -57,12 +58,12 @@ public:
     /**
      * NOTE: The returned pointer will become invalid as soon as the underlying vector changes.
      */
-    KTextEditor::Document *item(int row) const;
+    DocOrWidget item(int row) const;
 
     /**
      * Move the document to row 0.
      */
-    void raiseDocument(KTextEditor::Document *document);
+    void raiseDocument(DocOrWidget document);
 
     /*
      * Use this method to update all items.
@@ -99,5 +100,3 @@ private:
 QString longestCommonPrefix(std::vector<QString> const &strs);
 
 }
-
-#endif // KTEXTEDITOR_TAB_SWITCHER_FILES_MODEL_H
