@@ -13,7 +13,12 @@
 #include <KAboutData>
 #include <KDBusService>
 #include <KLocalizedString>
+
+// X11 startup handling
+#if __has_include(<KStartupInfo>)
 #include <KStartupInfo>
+#endif
+
 #include <KWindowSystem>
 #include <algorithm>
 
@@ -33,58 +38,40 @@
 
 #include "SingleApplication/SingleApplication"
 
-#ifndef Q_OS_WIN
-#include <unistd.h>
+#include "config-kate.h"
+
+#if HAVE_X11
+
+#include "kwindowsystem_version.h"
+#if KWINDOWSYSTEM_VERSION > QT_VERSION_CHECK(5, 101, 0)
+#include <KX11Extras>
+#endif
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+#include <private/qtx11extras_p.h>
+#else
+#include <QX11Info>
+#endif
 #endif
-#include <iostream>
 
 int main(int argc, char **argv)
 {
-#if !defined(Q_OS_WIN) && !defined(Q_OS_HAIKU)
-    // Prohibit using sudo or kdesu (but allow using the root user directly)
-    if (getuid() == 0) {
-        if (!qEnvironmentVariableIsEmpty("SUDO_USER")) {
-            std::cout << "Running Kate with sudo can cause bugs and expose you to security vulnerabilities. "
-                         "Instead use Kate normally and you will be prompted for elevated privileges when "
-                         "saving documents if needed."
-                      << std::endl;
-            return EXIT_FAILURE;
-        } else if (!qEnvironmentVariableIsEmpty("KDESU_USER")) {
-            std::cout << "Running Kate with kdesu can cause bugs and expose you to security vulnerabilities. "
-                         "Instead use Kate normally and you will be prompted for elevated privileges when "
-                         "saving documents if needed."
-                      << std::endl;
-            return EXIT_FAILURE;
+    /**
+     * fork into the background if we don't need to be blocking
+     * we need to do that early
+     */
+    bool detach = true;
+    for (int i = 1; i < argc; ++i) {
+        if (!strcmp(argv[i], "-b") || !strcmp(argv[i], "--block") || !strcmp(argv[i], "-i") || !strcmp(argv[i], "--stdin")) {
+            detach = false;
+            break;
         }
     }
-#endif
-    /**
-     * init resources from our static lib
-     */
-    Q_INIT_RESOURCE(kate);
-
-#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
-    /**
-     * enable high dpi support
-     */
-    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps, true);
-    QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling, true);
-#endif
 
     /**
-     * allow fractional scaling
-     * we only activate this on Windows, it seems to creates problems on unices
-     * (and there the fractional scaling with the QT_... env vars as set by KScreen works)
-     * see bug 416078
-     *
-     * we switched to Qt::HighDpiScaleFactorRoundingPolicy::RoundPreferFloor because of font rendering issues
-     * we follow what Krita does here, see https://invent.kde.org/graphics/krita/-/blob/master/krita/main.cc
-     * we raise the Qt requirement to  5.15 as it seems some patches went in after 5.14 that are needed
-     * see Krita comments, too
+     * Do all needed pre-application init steps, shared between Kate and KWrite
      */
-#if defined(Q_OS_WIN)
-    QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::RoundPreferFloor);
-#endif
+    KateApp::initPreApplicationCreation(detach);
 
     /**
      * Create application first
@@ -341,7 +328,17 @@ int main(int argc, char **argv)
         // prefer the Kate instance running on the current virtual desktop
         bool foundRunningService = false;
         if ((!force_new) && (serviceName.isEmpty())) {
-            const int desktopnumber = KWindowSystem::currentDesktop();
+            int desktopnumber = 0;
+#if HAVE_X11
+#if KWINDOWSYSTEM_VERSION > QT_VERSION_CHECK(5, 101, 0)
+            if (KWindowSystem::isPlatformX11()) {
+                desktopnumber = KX11Extras::currentDesktop();
+            }
+#else
+            desktopnumber = KWindowSystem::currentDesktop();
+#endif
+#endif
+
             for (int s = 0; s < kateServices.count(); s++) {
                 serviceName = kateServices[s];
 
@@ -490,7 +487,13 @@ int main(int argc, char **argv)
                                                                       QStringLiteral("/MainApplication"),
                                                                       QStringLiteral("org.kde.Kate.Application"),
                                                                       QStringLiteral("activate"));
-            activateMsg.setArguments({qEnvironmentVariable("XDG_ACTIVATION_TOKEN")});
+            if (KWindowSystem::isPlatformWayland()) {
+                activateMsg.setArguments({qEnvironmentVariable("XDG_ACTIVATION_TOKEN")});
+            } else if (KWindowSystem::isPlatformX11()) {
+#if HAVE_X11
+                activateMsg.setArguments({QString::fromUtf8(QX11Info::nextStartupId())});
+#endif
+            }
             QDBusConnection::sessionBus().call(activateMsg);
 
             // connect dbus signal
@@ -514,8 +517,10 @@ int main(int argc, char **argv)
             KDBusService dbusService(KDBusService::Multiple);
             dbusService.unregister();
 
+#if __has_include(<KStartupInfo>)
             // make the world happy, we are started, kind of...
             KStartupInfo::appStarted();
+#endif
 
             // We don't want the session manager to restart us on next login
             // if we block
