@@ -40,6 +40,10 @@ private Q_SLOTS:
     void testTabBarHidesShows();
     void testNewWindowHasSameGlobalOptions();
     void testBug465811();
+    void testBug465807();
+    void testBug465808();
+    void testNewViewCreatedIfViewNotinViewspace();
+    void testNewSessionClearsWindowWidgets();
 
 private:
     class QTemporaryDir *m_tempdir;
