@@ -5,6 +5,7 @@
 #include "diffwidget.h"
 #include "gitdiff.h"
 #include "gitprocess.h"
+#include "katemainwindow.h"
 #include "ktexteditor_utils.h"
 
 #include <QApplication>
@@ -18,6 +19,7 @@
 #include <QScrollBar>
 #include <QSyntaxHighlighter>
 #include <QTemporaryFile>
+#include <QTimer>
 #include <QToolBar>
 #include <QToolButton>
 
@@ -29,6 +31,66 @@
 #include <KSyntaxHighlighting/Repository>
 #include <KTextEditor/Editor>
 
+DiffWidget *DiffWidgetManager::existingDiffWidgetForParams(KateMainWindow *mw, const DiffParams &p)
+{
+    const auto widgets = mw->widgets();
+    for (auto widget : widgets) {
+        auto diffWidget = qobject_cast<DiffWidget *>(widget);
+        if (!diffWidget) {
+            continue;
+        }
+
+        if (diffWidget->m_params.arguments == p.arguments) {
+            return diffWidget;
+            break;
+        }
+    }
+    return nullptr;
+}
+
+void DiffWidgetManager::openDiff(const QByteArray &diff, DiffParams p, class KateMainWindow *mw)
+{
+    DiffWidget *existing = existingDiffWidgetForParams(mw, p);
+    if (!existing) {
+        existing = new DiffWidget(p, mw);
+        if (!p.tabTitle.isEmpty()) {
+            existing->setWindowTitle(p.tabTitle);
+        } else {
+            if (p.destFile.isEmpty())
+                existing->setWindowTitle(i18n("Diff %1", Utils::fileNameFromPath(p.srcFile)));
+            else
+                existing->setWindowTitle(i18n("Diff %1..%2", Utils::fileNameFromPath(p.srcFile), Utils::fileNameFromPath(p.destFile)));
+        }
+        existing->setWindowIcon(QIcon::fromTheme(QStringLiteral("text-x-patch")));
+        existing->openDiff(diff);
+        mw->addWidget(existing);
+    } else {
+        existing->clearData();
+        existing->m_params = p;
+        existing->openDiff(diff);
+        mw->activateWidget(existing);
+    }
+}
+
+void DiffWidgetManager::diffDocs(KTextEditor::Document *l, KTextEditor::Document *r, class KateMainWindow *mw)
+{
+    DiffParams p;
+    p.arguments = DiffWidget::diffDocsGitArgs(l, r);
+    DiffWidget *existing = existingDiffWidgetForParams(mw, p);
+    if (!existing) {
+        existing = new DiffWidget(p, mw);
+        existing->diffDocs(l, r);
+        existing->setWindowTitle(i18n("Diff %1 .. %2", l->documentName(), r->documentName()));
+        existing->setWindowIcon(QIcon::fromTheme(QStringLiteral("text-x-patch")));
+        mw->addWidget(existing);
+    } else {
+        existing->clearData();
+        existing->m_params = p;
+        existing->diffDocs(l, r);
+        mw->activateWidget(existing);
+    }
+}
+
 class Toolbar : public QToolBar
 {
     Q_OBJECT
@@ -67,6 +129,14 @@ public:
         m_showPrevHunk = addAction(QIcon::fromTheme(QStringLiteral("arrow-up")), QString());
         m_showPrevHunk->setToolTip(i18n("Jump to Previous Hunk"));
         connect(m_showPrevHunk, &QAction::triggered, this, &Toolbar::jumpToPrevHunk);
+
+        m_reload = addAction(QIcon::fromTheme(QStringLiteral("view-refresh")), QString());
+        m_reload->setToolTip(i18nc("Tooltip for a button, clicking the button reloads the diff", "Reload Diff"));
+        connect(m_reload, &QAction::triggered, this, &Toolbar::reload);
+
+        m_fullContext = addAction(QIcon::fromTheme(QStringLiteral("view-fullscreen")), QString());
+        m_fullContext->setToolTip(i18nc("Tooltip for a button", "Show diff with full context, not just the changed lines"));
+        connect(m_fullContext, &QAction::triggered, this, &Toolbar::showWithFullContext);
     }
 
     void setShowCommitActionVisible(bool vis)
@@ -81,12 +151,19 @@ public:
         return m_showCommitInfoAction->isChecked();
     }
 
+    void hideShowFullContext()
+    {
+        m_fullContext->setVisible(false);
+    }
+
 private:
     QAction *m_showCommitInfoAction;
     QAction *m_showNextFile;
     QAction *m_showPrevFile;
     QAction *m_showNextHunk;
     QAction *m_showPrevHunk;
+    QAction *m_reload;
+    QAction *m_fullContext;
 
 Q_SIGNALS:
     void showCommitInfoChanged(bool);
@@ -94,8 +171,23 @@ Q_SIGNALS:
     void jumpToPrevFile();
     void jumpToNextHunk();
     void jumpToPrevHunk();
+    void reload();
+    void showWithFullContext();
 };
 
+static void syncScroll(QPlainTextEdit *src, QPlainTextEdit *tgt)
+{
+    int srcValue = src->verticalScrollBar()->value();
+    const auto srcBlock = src->document()->findBlockByLineNumber(srcValue);
+    auto tgtBlock = tgt->document()->findBlockByLineNumber(srcValue);
+    if (srcBlock.blockNumber() == tgtBlock.blockNumber()) {
+        tgt->verticalScrollBar()->setValue(srcValue);
+    } else {
+        tgtBlock = tgt->document()->findBlockByNumber(srcBlock.blockNumber());
+        tgt->verticalScrollBar()->setValue(tgtBlock.firstLineNumber());
+    }
+}
+
 DiffWidget::DiffWidget(DiffParams p, QWidget *parent)
     : QWidget(parent)
     , m_left(new DiffEditor(p.flags, this))
@@ -120,17 +212,21 @@ DiffWidget::DiffWidget(DiffParams p, QWidget *parent)
     leftHl->setTheme(KTextEditor::Editor::instance()->theme());
     rightHl->setTheme(KTextEditor::Editor::instance()->theme());
 
-    connect(m_left->verticalScrollBar(), &QScrollBar::valueChanged, this, [this](int v) {
+    connect(m_left->verticalScrollBar(), &QScrollBar::valueChanged, this, [this](int) {
         if (m_stopScrollSync) {
             return;
         }
-        m_right->verticalScrollBar()->setValue(v);
+        m_stopScrollSync = true;
+        syncScroll(m_left, m_right);
+        m_stopScrollSync = false;
     });
-    connect(m_right->verticalScrollBar(), &QScrollBar::valueChanged, this, [this](int v) {
+    connect(m_right->verticalScrollBar(), &QScrollBar::valueChanged, this, [this](int) {
         if (m_stopScrollSync) {
             return;
         }
-        m_left->verticalScrollBar()->setValue(v);
+        m_stopScrollSync = true;
+        syncScroll(m_right, m_left);
+        m_stopScrollSync = false;
     });
 
     for (auto *e : {m_left, m_right}) {
@@ -155,10 +251,35 @@ DiffWidget::DiffWidget(DiffParams p, QWidget *parent)
     connect(m_toolbar, &Toolbar::jumpToPrevFile, this, &DiffWidget::jumpToPrevFile);
     connect(m_toolbar, &Toolbar::jumpToNextHunk, this, &DiffWidget::jumpToNextHunk);
     connect(m_toolbar, &Toolbar::jumpToPrevHunk, this, &DiffWidget::jumpToPrevHunk);
+    connect(m_toolbar, &Toolbar::reload, this, &DiffWidget::runGitDiff);
+    connect(m_toolbar, &Toolbar::showWithFullContext, this, &DiffWidget::showWithFullContext);
 
     KSharedConfig::Ptr config = KSharedConfig::openConfig();
     KConfigGroup cgGeneral = KConfigGroup(config, "General");
     handleStyleChange(cgGeneral.readEntry("Diff Show Style", (int)SideBySide));
+    // clear, after handleStyleChange there might be "no differences found" text
+    m_left->clear();
+    m_right->clear();
+}
+
+DiffWidget::~DiffWidget()
+{
+    // if there are any living processes, disconnect them now before we get destroyed
+    for (QObject *child : children()) {
+        QProcess *p = qobject_cast<QProcess *>(child);
+        if (p) {
+            disconnect(p, nullptr, nullptr, nullptr);
+        }
+    }
+}
+
+void DiffWidget::showEvent(QShowEvent *e)
+{
+    if (!m_blockShowEvent && m_params.flags & DiffParams::ReloadOnShow) {
+        runGitDiff();
+    }
+
+    QWidget::showEvent(e);
 }
 
 void DiffWidget::handleStyleChange(int newStyle)
@@ -223,6 +344,24 @@ void DiffWidget::handleStageUnstage_sideBySide(DiffEditor *e, int startLine, int
                 break;
             }
         }
+
+        // If the selection by user isn't exact i.e.,
+        // the start line isn't a changed line but just a context line
+        if (diffLine == -1) {
+            for (auto vToD : std::as_const(m_lineToRawDiffLine)) {
+                // find the closes line to start line
+                if (vToD.line > startLine && vToD.added == added) {
+                    // check if the found line is in selection range
+                    if (vToD.line <= endLine) {
+                        // adjust startline
+                        startLine = vToD.line;
+                        diffLine = vToD.diffLine;
+                    }
+                    break;
+                }
+            }
+        }
+
     } else if (actionType == DiffEditor::Hunk) {
         // find the first hunk smaller than/equal this line
         for (auto it = m_lineToDiffHunkLine.crbegin(); it != m_lineToDiffHunkLine.crend(); ++it) {
@@ -293,6 +432,9 @@ void DiffWidget::applyDiff(const QString &diff, ApplyFlags flags)
             onError(git->readAllStandardError(), git->exitCode());
         } else {
             runGitDiff();
+            if (m_params.updateStatusCallback) {
+                m_params.updateStatusCallback();
+            }
         }
         delete file;
         git->deleteLater();
@@ -307,27 +449,32 @@ void DiffWidget::runGitDiff()
     if (workingDir.isEmpty() || arguments.isEmpty()) {
         return;
     }
+    m_blockShowEvent = true;
 
-    auto leftState = m_left->saveState();
-    auto rightState = m_right->saveState();
+    const int lf = m_left->firstVisibleBlockNumber();
+    const int rf = m_right->firstVisibleBlockNumber();
 
     QProcess *git = new QProcess(this);
     setupGitProcess(*git, workingDir, arguments);
-    connect(git, &QProcess::finished, this, [=](int exitCode, QProcess::ExitStatus es) {
+    connect(git, &QProcess::finished, this, [=](int, QProcess::ExitStatus) {
         const auto params = m_params;
         clearData();
         m_params = params;
-        m_left->setUpdatesEnabled(false);
-        m_right->setUpdatesEnabled(false);
-        if (es != QProcess::NormalExit || exitCode != 0) {
+        const auto out = git->readAllStandardOutput();
+        const auto err = git->readAllStandardError();
+        if (!err.isEmpty()) {
             onError(git->readAllStandardError(), git->exitCode());
         } else {
-            openDiff(git->readAllStandardOutput());
-            m_left->restoreState(leftState);
-            m_right->restoreState(rightState);
+            openDiff(out);
+            QMetaObject::invokeMethod(
+                this,
+                [this, lf, rf] {
+                    m_left->scrollToBlock(lf);
+                    m_right->scrollToBlock(rf);
+                },
+                Qt::QueuedConnection);
         }
-        m_left->setUpdatesEnabled(true);
-        m_right->setUpdatesEnabled(true);
+        m_blockShowEvent = false;
         git->deleteLater();
     });
     startHostProcess(*git, QProcess::ReadOnly);
@@ -346,6 +493,45 @@ void DiffWidget::clearData()
     m_commitInfo->hide();
 }
 
+QStringList DiffWidget::diffDocsGitArgs(KTextEditor::Document *l, KTextEditor::Document *r)
+{
+    const QString left = l->url().toLocalFile();
+    const QString right = r->url().toLocalFile();
+    return {QStringLiteral("diff"), QStringLiteral("--no-color"), QStringLiteral("--no-index"), left, right};
+}
+
+void DiffWidget::showWithFullContext()
+{
+    if (m_params.arguments.size() < 1) {
+        return;
+    }
+
+    const int lineNo = m_left->firstVisibleLineNumber();
+
+    int idx = m_params.arguments.indexOf(QLatin1String("--"));
+    if (idx != -1) {
+        m_params.arguments.insert(idx, QLatin1String("-U5000"));
+    } else if (m_params.arguments.size() == 1) {
+        m_params.arguments << QLatin1String("-U5000");
+    } else {
+        m_params.arguments.insert(2, QLatin1String("-U5000"));
+    }
+    runGitDiff();
+
+    // After the diff runs and we show the result, try to take the user back to where he was
+    QPointer<QTimer> delayedSlotTrigger = new QTimer(this);
+    delayedSlotTrigger->setSingleShot(true);
+    delayedSlotTrigger->setInterval(10);
+    delayedSlotTrigger->callOnTimeout(this, [this, lineNo, delayedSlotTrigger] {
+        if (delayedSlotTrigger) {
+            m_left->scrollToLineNumber(lineNo);
+            m_toolbar->hideShowFullContext();
+            delete delayedSlotTrigger;
+        }
+    });
+    connect(m_left, &QPlainTextEdit::textChanged, delayedSlotTrigger, qOverload<>(&QTimer::start));
+}
+
 void DiffWidget::diffDocs(KTextEditor::Document *l, KTextEditor::Document *r)
 {
     clearData();
@@ -359,25 +545,21 @@ void DiffWidget::diffDocs(KTextEditor::Document *l, KTextEditor::Document *r)
         rightHl->setDefinition(repo.definitionForMimeType(r->mimeType()));
     }
 
-    const QString left = l->url().toLocalFile();
-    const QString right = r->url().toLocalFile();
+    QProcess git;
+    if (!setupGitProcess(git, qApp->applicationDirPath(), diffDocsGitArgs(l, r))) {
+        Utils::showMessage(
+            i18n("<b>git</b> not found. Git is needed to diff the documents. If git is already installed, make sure it is your PATH variable. See "
+                 "https://git-scm.com/downloads"),
+            gitIcon(),
+            i18n("Diff"),
+            MessageType::Error);
+        return;
+    }
 
-    QPointer<QProcess> git = new QProcess(this);
-    setupGitProcess(*git, qApp->applicationDirPath(), {QStringLiteral("diff"), QStringLiteral("--no-index"), left, right});
-
-    connect(git, &QProcess::readyReadStandardOutput, this, [this, git]() {
-        onTextReceived(git->readAllStandardOutput());
-    });
-    connect(git, &QProcess::readyReadStandardError, this, [this, git]() {
-        onError(git->readAllStandardError(), -1);
-    });
-    connect(git, &QProcess::finished, this, [this, git] {
-        git->deleteLater();
-        if (git->exitStatus() != QProcess::NormalExit) {
-            onError(git->readAllStandardError(), git->exitCode());
-        }
-    });
-    git->start();
+    m_params.arguments = git.arguments();
+    m_params.flags.setFlag(DiffParams::ReloadOnShow);
+    m_params.workingDir = git.workingDirectory();
+    runGitDiff();
 }
 
 static void balanceHunkLines(QStringList &left, QStringList &right, int &lineA, int &lineB, QVector<int> &lineNosA, QVector<int> &lineNosB)
@@ -558,20 +740,28 @@ void DiffWidget::parseAndShowDiff(const QByteArray &raw)
     for (int i = 0; i < text.size(); ++i) {
         const QString &line = text.at(i);
         auto match = DIFF_FILENAME_RE.match(line);
-        if (match.hasMatch() && i + 1 < text.size()) {
-            srcFile = match.captured(1);
+        if ((match.hasMatch() || line == QStringLiteral("--- /dev/null")) && i + 1 < text.size()) {
+            srcFile = match.hasMatch() ? match.captured(1) : QString();
             mimeTypes.insert(QMimeDatabase().mimeTypeForFile(match.captured(1), QMimeDatabase::MatchExtension).name());
             auto match = DIFF_FILENAME_RE.match(text.at(i + 1));
 
-            if (match.hasMatch()) {
-                tgtFile = match.captured(1);
+            if (match.hasMatch() || text.at(i + 1) == QStringLiteral("--- /dev/null")) {
+                tgtFile = match.hasMatch() ? match.captured(1) : QString();
                 mimeTypes.insert(QMimeDatabase().mimeTypeForFile(match.captured(1), QMimeDatabase::MatchExtension).name());
             }
             i++;
 
             if (m_params.flags.testFlag(DiffParams::ShowFileName)) {
-                left.append(Utils::fileNameFromPath(srcFile));
-                right.append(Utils::fileNameFromPath(tgtFile));
+                if (srcFile.isEmpty() && !tgtFile.isEmpty()) {
+                    left.append(QLatin1String("---"));
+                    right.append(i18n("New file %1", Utils::fileNameFromPath(tgtFile)));
+                } else if (!srcFile.isEmpty() && tgtFile.isEmpty()) {
+                    left.append(i18n("Deleted file %1", Utils::fileNameFromPath(srcFile)));
+                    right.append(QLatin1String("+++"));
+                } else {
+                    left.append(Utils::fileNameFromPath(srcFile));
+                    right.append(Utils::fileNameFromPath(tgtFile));
+                }
                 Q_ASSERT(left.size() == right.size() && lineA == lineB);
                 linesWithFileName.append(lineA);
                 lineNumsA.append(-1);
@@ -751,19 +941,25 @@ void DiffWidget::parseAndShowDiffUnified(const QByteArray &raw)
     for (int i = 0; i < text.size(); ++i) {
         const QString &line = text.at(i);
         auto match = DIFF_FILENAME_RE.match(line);
-        if (match.hasMatch() && i + 1 < text.size()) {
-            srcFile = match.captured(1);
+        if ((match.hasMatch() || line == QStringLiteral("--- /dev/null")) && i + 1 < text.size()) {
+            srcFile = match.hasMatch() ? match.captured(1) : QString();
             mimeTypes.insert(QMimeDatabase().mimeTypeForFile(match.captured(1), QMimeDatabase::MatchExtension).name());
             auto match = DIFF_FILENAME_RE.match(text.at(i + 1));
 
-            if (match.hasMatch()) {
-                tgtFile = match.captured(1);
+            if (match.hasMatch() || text.at(i + 1) == QStringLiteral("--- /dev/null")) {
+                tgtFile = match.hasMatch() ? match.captured(1) : QString();
                 mimeTypes.insert(QMimeDatabase().mimeTypeForFile(match.captured(1), QMimeDatabase::MatchExtension).name());
             }
             i++;
 
             if (m_params.flags.testFlag(DiffParams::ShowFileName)) {
-                lines.append(QStringLiteral("%1 → %2").arg(Utils::fileNameFromPath(srcFile), Utils::fileNameFromPath(tgtFile)));
+                if (srcFile.isEmpty() && !tgtFile.isEmpty()) {
+                    lines.append(i18n("New file %1", Utils::fileNameFromPath(tgtFile)));
+                } else if (!srcFile.isEmpty() && tgtFile.isEmpty()) {
+                    lines.append(i18n("Deleted file %1", Utils::fileNameFromPath(srcFile)));
+                } else if (!srcFile.isEmpty() && !tgtFile.isEmpty()) {
+                    lines.append(QStringLiteral("%1 → %2").arg(Utils::fileNameFromPath(srcFile), Utils::fileNameFromPath(tgtFile)));
+                }
                 lineNumsA.append(-1);
                 lineNumsB.append(-1);
                 linesWithFileName.append(lineNo);
@@ -902,33 +1098,49 @@ void DiffWidget::openDiff(const QByteArray &raw)
         m_toolbar->setShowCommitActionVisible(false);
     }
 
+    // Fallback to raw mode if parsing fails
+    auto fallback = [&] {
+        handleStyleChange(Raw);
+        m_left->setPlainText(QString::fromUtf8(raw));
+        leftHl->setDefinition(KTextEditor::Editor::instance()->repository().definitionForName(QStringLiteral("Diff")));
+    };
+
     if (m_style == SideBySide) {
         parseAndShowDiff(raw);
+        if (m_left->document()->isEmpty() && m_right->document()->isEmpty() && !raw.isEmpty()) {
+            fallback();
+        }
     } else if (m_style == Unified) {
         parseAndShowDiffUnified(raw);
+        if (m_left->document()->isEmpty() && !raw.isEmpty()) {
+            fallback();
+        }
     } else if (m_style == Raw) {
         m_left->setPlainText(QString::fromUtf8(raw));
         leftHl->setDefinition(KTextEditor::Editor::instance()->repository().definitionForName(QStringLiteral("Diff")));
     }
     m_rawDiff = raw;
-    m_left->verticalScrollBar()->setValue(0);
-    m_right->verticalScrollBar()->setValue(0);
-}
 
-void DiffWidget::onTextReceived(const QByteArray &raw)
-{
-    if (m_style == SideBySide) {
-        parseAndShowDiff(raw);
-    } else if (m_style == Unified) {
-        parseAndShowDiffUnified(raw);
-    } else if (m_style == Raw) {
-        m_left->appendPlainText(QString::fromUtf8(raw));
+    if (m_rawDiff.isEmpty()) {
+        m_left->setPlainText(i18n("No differences found"));
+        m_right->setPlainText(i18n("No differences found"));
     }
-    m_rawDiff += raw;
+
+    QMetaObject::invokeMethod(
+        this,
+        [this] {
+            m_left->verticalScrollBar()->setValue(0);
+            m_right->verticalScrollBar()->setValue(0);
+            m_blockShowEvent = false;
+        },
+        Qt::QueuedConnection);
 }
 
-void DiffWidget::onError(const QByteArray & /*error*/, int /*code*/)
+void DiffWidget::onError(const QByteArray &error, int)
 {
+    if (!error.isEmpty()) {
+        Utils::showMessage(QString::fromUtf8(error), gitIcon(), i18n("Diff"), MessageType::Warning);
+    }
     //     printf("Got error: \n%s\n==============\n", error.constData());
 }
 
@@ -970,9 +1182,9 @@ void DiffWidget::jumpToNextFile()
     }
 
     QScopedValueRollback r(m_stopScrollSync, true);
-    m_left->scrollToBlock(nextFileLineNo);
+    m_left->scrollToBlock(nextFileLineNo, true);
     if (m_style == SideBySide) {
-        m_right->scrollToBlock(nextFileLineNo);
+        m_right->scrollToBlock(nextFileLineNo, true);
     }
 }
 
@@ -988,9 +1200,9 @@ void DiffWidget::jumpToPrevFile()
     }
 
     QScopedValueRollback r(m_stopScrollSync, true);
-    m_left->scrollToBlock(prevFileLineNo);
+    m_left->scrollToBlock(prevFileLineNo, true);
     if (m_style == SideBySide) {
-        m_right->scrollToBlock(prevFileLineNo);
+        m_right->scrollToBlock(prevFileLineNo, true);
     }
 }
 
@@ -1006,9 +1218,9 @@ void DiffWidget::jumpToNextHunk()
     }
 
     QScopedValueRollback r(m_stopScrollSync, true);
-    m_left->scrollToBlock(nextHunkLineNo);
+    m_left->scrollToBlock(nextHunkLineNo, true);
     if (m_style == SideBySide) {
-        m_right->scrollToBlock(nextHunkLineNo);
+        m_right->scrollToBlock(nextHunkLineNo, true);
     }
 }
 
@@ -1024,9 +1236,9 @@ void DiffWidget::jumpToPrevHunk()
     }
 
     QScopedValueRollback r(m_stopScrollSync, true);
-    m_left->scrollToBlock(prevHunkLineNo);
+    m_left->scrollToBlock(prevHunkLineNo, true);
     if (m_style == SideBySide) {
-        m_right->scrollToBlock(prevHunkLineNo);
+        m_right->scrollToBlock(prevHunkLineNo, true);
     }
 }
 
