@@ -31,8 +31,6 @@
 #include <KSharedConfig>
 #include <kwidgetsaddons_version.h>
 
-#include <kwidgetsaddons_version.h>
-
 #include <QCheckBox>
 #include <QComboBox>
 #include <QDesktopServices>
@@ -49,18 +47,6 @@
 #include <QTimer>
 #include <QVBoxLayout>
 
-static void ignoreSizePolicyForStackedWidget(QObject *p)
-{
-    const auto widgets = p->findChildren<QWidget *>();
-    for (auto w : widgets) {
-        if (w && qstrcmp(w->metaObject()->className(), "KPageStackedWidget") == 0) {
-            w->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
-            return;
-        }
-    }
-    qWarning() << "Failed to ignoreSizePolicy, there might be some sizing issues in config";
-}
-
 KateConfigDialog::KateConfigDialog(KateMainWindow *parent)
     : KPageDialog(parent)
     , m_mainWindow(parent)
@@ -89,6 +75,8 @@ KateConfigDialog::KateConfigDialog(KateMainWindow *parent)
     if (KateApp::isKate()) {
 #if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 94, 0)
         setFaceType(KPageDialog::FlatList);
+#else
+        setFaceType(KPageDialog::List);
 #endif
     }
     else {
@@ -131,13 +119,6 @@ KateConfigDialog::KateConfigDialog(KateMainWindow *parent)
     connect(buttonBox()->button(QDialogButtonBox::Apply), &QPushButton::clicked, this, &KateConfigDialog::slotApply);
     connect(buttonBox()->button(QDialogButtonBox::Help), &QPushButton::clicked, this, &KateConfigDialog::slotHelp);
 
-    button(QDialogButtonBox::Ok)->hide();
-    button(QDialogButtonBox::Cancel)->setText(i18n("Close"));
-    button(QDialogButtonBox::Apply)->setText(i18n("Save"));
-
-    // Ignore size policy so that it doesn't try to grow bigger than main window
-    ignoreSizePolicyForStackedWidget(this);
-
     // set focus on next iteration of event loop, doesn't work directly for some reason
     QMetaObject::invokeMethod(
         this,
@@ -277,7 +258,7 @@ void KateConfigDialog::onSearchTextChanged()
         QModelIndex current;
         for (int i = 0; i < model->rowCount(); ++i) {
             const auto itemName = model->index(i, 0).data().toString();
-            m_sideBar->setRowHidden(i, pagesToHide.contains(itemName));
+            m_sideBar->setRowHidden(i, pagesToHide.contains(itemName) && !itemName.contains(text, Qt::CaseInsensitive));
             if (!text.isEmpty() && !m_sideBar->isRowHidden(i) && !current.isValid()) {
                 current = model->index(i, 0);
             }
@@ -340,14 +321,16 @@ void KateConfigDialog::onSearchTextChanged()
     }
 }
 
-KateConfigDialog *KateConfigDialog::widget(KateMainWindow *mw)
+QSize KateConfigDialog::sizeHint() const
 {
-    static QPointer<KateConfigDialog> dlg = nullptr;
-    if (!dlg) {
-        dlg = new KateConfigDialog(mw);
-        return dlg;
-    }
-    return dlg;
+    // start with a bit enlarged default size hint to minimize changes of useless scrollbars
+    QSize size = KPageDialog::sizeHint() * 1.3;
+
+    // enlarge it to half of the main window size, if that is larger
+    size = size.expandedTo(m_mainWindow->size() * 0.5);
+
+    // return bounded size to available real screen space
+    return size.boundedTo(screen()->availableSize() * 0.9);
 }
 
 void KateConfigDialog::addBehaviorPage()
@@ -369,6 +352,15 @@ void KateConfigDialog::addBehaviorPage()
     QVBoxLayout *vbox = new QVBoxLayout;
     layout->addWidget(buttonGroup);
 
+    // shall we try to behave like some SDI application
+    m_sdiMode = new QCheckBox(i18n("Open each document in its own window"), buttonGroup);
+    m_sdiMode->setChecked(cgGeneral.readEntry("SDI Mode", false));
+    m_sdiMode->setToolTip(
+        i18n("If enabled, each document will be opened in its own window. "
+             "If not enabled, each document will be opened in a new tab in the current window."));
+    connect(m_sdiMode, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
+    vbox->addWidget(m_sdiMode);
+
     QHBoxLayout *hlayout = nullptr;
     QLabel *label = nullptr;
 
@@ -400,8 +392,7 @@ void KateConfigDialog::addBehaviorPage()
     // modified files notification
     m_modNotifications = new QCheckBox(i18n("Use a separate &dialog for handling externally modified files"), buttonGroup);
     m_modNotifications->setChecked(m_mainWindow->modNotificationEnabled());
-    m_modNotifications->setCursor(Qt::WhatsThisCursor);
-    m_modNotifications->setWhatsThis(
+    m_modNotifications->setToolTip(
         i18n("If enabled, a modal dialog will be used to show all of the modified files. "
              "If not enabled, you will be individually asked what to do for each modified file "
              "only when that file's view receives focus."));
@@ -416,8 +407,7 @@ void KateConfigDialog::addBehaviorPage()
         QVBoxLayout *vbox = new QVBoxLayout;
         m_syncSectionSizeWithSidebarTabs = new QCheckBox(i18n("Sync section size with tab positions"), buttonGroup);
         m_syncSectionSizeWithSidebarTabs->setChecked(cgGeneral.readEntry("Sync section size with tab positions", false));
-        m_syncSectionSizeWithSidebarTabs->setCursor(Qt::WhatsThisCursor);
-        m_syncSectionSizeWithSidebarTabs->setWhatsThis(
+        m_syncSectionSizeWithSidebarTabs->setToolTip(
             i18n("When enabled the section size will be determined by the position of the tabs.\n"
                  "This option does not affect the bottom sidebar."));
         connect(m_syncSectionSizeWithSidebarTabs, &QCheckBox::toggled, this, &KateConfigDialog::slotChanged);
@@ -650,9 +640,9 @@ void KateConfigDialog::addPluginsPage()
     vlayout->setContentsMargins(0, 0, 0, 0);
     vlayout->setSpacing(0);
 
-    KateConfigPluginPage *configPluginPage = new KateConfigPluginPage(page, this);
-    vlayout->addWidget(configPluginPage);
-    connect(configPluginPage, &KateConfigPluginPage::changed, this, &KateConfigDialog::slotChanged);
+    m_configPluginPage = new KateConfigPluginPage(page, this);
+    vlayout->addWidget(m_configPluginPage);
+    connect(m_configPluginPage, &KateConfigPluginPage::changed, this, &KateConfigDialog::slotChanged);
 
     auto item = addScrollablePage(page, i18n("Plugins"));
     m_allPages.insert(item);
@@ -750,8 +740,15 @@ void KateConfigDialog::slotApply()
 
     // if data changed apply the kate app stuff
     if (m_dataChanged) {
+        // apply plugin load state changes
+        if (m_configPluginPage) {
+            m_configPluginPage->slotApply();
+        }
+
         KConfigGroup cg(config, "General");
 
+        cg.writeEntry("SDI Mode", m_sdiMode->isChecked());
+
         // only there for kate
         if (m_syncSectionSizeWithSidebarTabs) {
             cg.writeEntry("Sync section size with tab positions", m_syncSectionSizeWithSidebarTabs->isChecked());
@@ -864,8 +861,6 @@ void KateConfigDialog::slotApply()
 
     m_dataChanged = false;
     buttonBox()->button(QDialogButtonBox::Apply)->setEnabled(false);
-
-    Q_EMIT saved();
 }
 
 void KateConfigDialog::slotChanged()
