@@ -10,6 +10,7 @@
 // BEGIN Includes
 #include "katemainwindow.h"
 
+#include "diagnostics/diagnosticview.h"
 #include "filehistorywidget.h"
 #include "kateapp.h"
 #include "kateconfigdialog.h"
@@ -31,21 +32,12 @@
 
 #include <KActionCollection>
 #include <KActionMenu>
-#include <KApplicationTrader>
 #include <KColorSchemeManager>
 #include <KConfigGroup>
 #include <KEditToolBar>
 #include <KFileItem>
 #include <KHelpClient>
-#include <KIO/ApplicationLauncherJob>
 #include <KIO/Job>
-#include <kio_version.h>
-#include <kwidgetsaddons_version.h>
-#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
-#include <KIO/JobUiDelegateFactory>
-#else
-#include <KIO/JobUiDelegate>
-#endif
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KMultiTabBar>
@@ -60,6 +52,7 @@
 #include <KWindowConfig>
 #include <KXMLGUIFactory>
 #include <kconfigwidgets_version.h>
+#include <kwidgetsaddons_version.h>
 
 #include <QApplication>
 #include <QDir>
@@ -112,7 +105,7 @@ QSize KateContainerStackedLayout::minimumSize() const
     return QStackedLayout::minimumSize();
 }
 
-KateMainWindow::KateMainWindow(KConfig *sconfig, const QString &sgroup)
+KateMainWindow::KateMainWindow(KConfig *sconfig, const QString &sgroup, bool userTriggered)
     : KateMDI::MainWindow(nullptr)
     , m_modignore(false)
     , m_wrapper(new KTextEditor::MainWindow(this))
@@ -129,7 +122,7 @@ KateMainWindow::KateMainWindow(KConfig *sconfig, const QString &sgroup)
     setupImportantActions();
 
     // setup the most important widgets
-    setupMainWindow();
+    setupMainWindow(sconfig);
 
     // setup the actions
     setupActions();
@@ -157,7 +150,7 @@ KateMainWindow::KateMainWindow(KConfig *sconfig, const QString &sgroup)
 
     readOptions();
 
-    if (sconfig) {
+    if (sconfig && !userTriggered) {
         m_viewManager->restoreViewConfiguration(KConfigGroup(sconfig, sgroup));
     }
 
@@ -199,9 +192,13 @@ KateMainWindow::~KateMainWindow()
     // close all documents not visible in other windows, we did ask for permission in queryClose
     if (winClosesDocuments()) {
         auto docs = KateApp::self()->documentManager()->documentList();
+        const bool canStash = KateApp::self()->stashManager()->canStash();
         docs.erase(std::remove_if(docs.begin(),
                                   docs.end(),
-                                  [this](auto doc) {
+                                  [this, canStash](auto doc) {
+                                      if (canStash && (doc->isModified() || doc->url().isEmpty())) {
+                                          return true;
+                                      }
                                       return KateApp::self()->documentVisibleInOtherWindows(doc, this);
                                   }),
                    docs.end());
@@ -284,7 +281,7 @@ void KateMainWindow::setupImportantActions()
     hamburgerMenu->setShowMenuBarAction(m_paShowMenuBar);
 }
 
-void KateMainWindow::setupMainWindow()
+void KateMainWindow::setupMainWindow(KConfig *sconfig)
 {
     m_viewManager = new KateViewManager(centralWidget(), this);
     centralWidget()->layout()->addWidget(m_viewManager);
@@ -294,6 +291,11 @@ void KateMainWindow::setupMainWindow()
     centralWidget()->layout()->addWidget(m_bottomViewBarContainer);
     m_bottomContainerStack = new KateContainerStackedLayout(m_bottomViewBarContainer);
 
+    if (KateApp::isKWrite()) {
+        // Kwrite has nothing other than the view manager
+        return;
+    }
+
     /**
      * create generic output tool view
      * is used to display output of e.g. plugins
@@ -304,6 +306,14 @@ void KateMainWindow::setupMainWindow()
                                       QIcon::fromTheme(QStringLiteral("output_win")),
                                       i18n("Output"));
     m_outputView = new KateOutputView(this, m_toolViewOutput, toolviewToggleButton(static_cast<KateMDI::ToolView *>(m_toolViewOutput)));
+
+    m_toolViewDiags = createToolView(nullptr /* toolview has no plugin it belongs to */,
+                                     QStringLiteral("diagnostics"),
+                                     KTextEditor::MainWindow::Bottom,
+                                     QIcon::fromTheme(QStringLiteral("dialog-warning-symbolic")),
+                                     i18n("Diagnostics"));
+    m_diagView = new DiagnosticsView(m_toolViewDiags, this, toolviewToggleButton(static_cast<KateMDI::ToolView *>(m_toolViewDiags)));
+    m_diagView->readSessionConfig(KConfigGroup(sconfig, "Kate Diagnostics"));
 }
 
 void KateMainWindow::setupActions()
@@ -599,12 +609,15 @@ bool KateMainWindow::queryClose_internal(KTextEditor::Document *doc, KateMainWin
     std::vector<KTextEditor::Document *> modifiedDocuments = KateApp::self()->documentManager()->modifiedDocumentList();
 
     // filter out what the stashManager will itself stash
-    modifiedDocuments.erase(std::remove_if(modifiedDocuments.begin(),
-                                           modifiedDocuments.end(),
-                                           [](auto doc) {
-                                               return KateApp::self()->stashManager()->willStashDoc(doc);
-                                           }),
-                            modifiedDocuments.end());
+    const bool canStash = KateApp::self()->stashManager()->canStash();
+    if (canStash) {
+        modifiedDocuments.erase(std::remove_if(modifiedDocuments.begin(),
+                                               modifiedDocuments.end(),
+                                               [](auto doc) {
+                                                   return KateApp::self()->stashManager()->willStashDoc(doc);
+                                               }),
+                                modifiedDocuments.end());
+    }
 
     // do we want to ignore some document?
     if (doc) {
@@ -615,7 +628,10 @@ bool KateMainWindow::queryClose_internal(KTextEditor::Document *doc, KateMainWin
     if (win) {
         modifiedDocuments.erase(std::remove_if(modifiedDocuments.begin(),
                                                modifiedDocuments.end(),
-                                               [win](auto doc) {
+                                               [win, canStash](auto doc) {
+                                                   if (canStash && (doc->isModified() || doc->url().isEmpty())) {
+                                                       return true;
+                                                   }
                                                    return KateApp::self()->documentVisibleInOtherWindows(doc, win);
                                                }),
                                 modifiedDocuments.end());
@@ -663,9 +679,13 @@ bool KateMainWindow::queryClose()
     return false;
 }
 
-void KateMainWindow::newWindow()
+KateMainWindow *KateMainWindow::newWindow() const
 {
-    KateApp::self()->newMainWindow(KateApp::self()->sessionManager()->activeSession()->config());
+    // create new window with current session
+    // derive size from current one
+    auto win = KateApp::self()->newMainWindow(KateApp::self()->sessionManager()->activeSession()->config(), {}, true);
+    win->resize(size());
+    return win;
 }
 
 void KateMainWindow::slotEditToolbars()
@@ -957,31 +977,20 @@ void KateMainWindow::slotConfigure()
 
 bool KateMainWindow::showPluginConfigPage(KTextEditor::Plugin *configpageinterface, int id)
 {
-    KateConfigDialog *dlg = KateConfigDialog::widget(this);
-    if (dlg->mainwWindow() != this) {
-        dlg->reject();
-        if (dlg->mainwWindow())
-            dlg->mainwWindow()->removeWidget(dlg);
-        delete dlg;
-        dlg = KateConfigDialog::widget(this);
-    }
+    KateConfigDialog *dlg = new KateConfigDialog(this);
+
     if (configpageinterface) {
         dlg->showAppPluginPage(configpageinterface, id);
     }
 
-    activateWidget(dlg);
-
-    connect(dlg, &QDialog::rejected, this, [this, dlg]() {
-        removeWidget(dlg);
-    });
-
-    connect(dlg, &KateConfigDialog::saved, this, [this]() {
+    if (dlg->exec() == QDialog::Accepted) {
         m_fileOpenRecent->setMaxItems(KateConfigDialog::recentFilesMaxCount());
-        // gui (toolbars...) needs to be updated, because
-        // of possible changes that the configure dialog
-        // could have done on it, specially for plugins.
-        m_viewManager->replugActiveView();
-    });
+    }
+    delete dlg;
+
+    m_viewManager->replugActiveView(); // gui (toolbars...) needs to be updated, because
+    // of possible changes that the configure dialog
+    // could have done on it, specially for plugins.
 
     return true;
 }
@@ -999,50 +1008,32 @@ QUrl KateMainWindow::activeDocumentUrl()
 
 void KateMainWindow::mSlotFixOpenWithMenu()
 {
-    // dh: in bug #307699, this slot is called when launching the Kate application
-    // unfortunately, no one ever could reproduce except users.
     KTextEditor::View *activeView = m_viewManager->activeView();
     if (!activeView) {
         return;
     }
 
-    // cleanup menu
-    QMenu *menu = documentOpenWith->menu();
-    menu->clear();
-
-    // get a list of appropriate services.
-    QMimeDatabase db;
-    QMimeType mime = db.mimeTypeForName(activeView->document()->mimeType());
-    // qCDebug(LOG_KATE) << "mime type: " << mime.name();
-
-    QAction *a = nullptr;
-    const KService::List offers = KApplicationTrader::queryByMimeType(mime.name());
-    // add all default open-with-actions except "Kate"
-    for (const auto &service : offers) {
-        if (service->name() == QLatin1String("Kate")) {
-            continue;
-        }
-        a = menu->addAction(QIcon::fromTheme(service->icon()), service->name());
-        a->setData(service->entryPath());
+    KTextEditor::Document *doc = activeView->document();
+    if (!doc) {
+        return;
     }
-    // append "Other..." to call the KDE "open with" dialog.
-    a = documentOpenWith->menu()->addAction(i18n("&Other..."));
-    a->setData(QString());
+
+    KateFileActions::prepareOpenWithMenu(doc->url(), documentOpenWith->menu());
 }
 
 void KateMainWindow::slotOpenWithMenuAction(QAction *a)
 {
-    const QList<QUrl> list({m_viewManager->activeView()->document()->url()});
-    KService::Ptr app = KService::serviceByDesktopPath(a->data().toString());
-    // If app is null, ApplicationLauncherJob will invoke the open-with dialog
-    auto *job = new KIO::ApplicationLauncherJob(app);
-    job->setUrls(list);
-#if KIO_VERSION >= QT_VERSION_CHECK(5, 98, 0)
-    job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, this));
-#else
-    job->setUiDelegate(new KIO::JobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, this));
-#endif
-    job->start();
+    auto activeView = m_viewManager->activeView();
+    if (!activeView) {
+        return;
+    }
+
+    auto doc = activeView->document();
+    if (!doc) {
+        return;
+    }
+
+    KateFileActions::showOpenWithMenu(this, doc->url(), a);
 }
 
 void KateMainWindow::pluginHelp()
@@ -1216,6 +1207,11 @@ void KateMainWindow::saveGlobalProperties(KConfig *sessionConfig)
 
     // save plugin config !!
     KateApp::self()->pluginManager()->writeConfig(sessionConfig);
+
+    if (m_diagView) {
+        KConfigGroup cg(sessionConfig, "Kate Diagnostics");
+        m_diagView->writeSessionConfig(cg);
+    }
 }
 
 void KateMainWindow::saveWindowConfig(const KConfigGroup &_config)
@@ -1334,8 +1330,13 @@ bool KateMainWindow::event(QEvent *e)
         QKeyEvent *k = static_cast<QKeyEvent *>(e);
         Q_EMIT unhandledShortcutOverride(k);
 
-        if (k->key() == Qt::Key_Escape && k->modifiers() == Qt::NoModifier && !m_toolViewOutput->isHidden()) {
-            hideToolView(m_toolViewOutput);
+        if (KateApp::isKate() && k->key() == Qt::Key_Escape && k->modifiers() == Qt::NoModifier) {
+            if (!m_toolViewOutput->isHidden()) {
+                hideToolView(m_toolViewOutput);
+            }
+            if (!m_toolViewDiags->isHidden()) {
+                hideToolView(m_toolViewDiags);
+            }
         }
     }
 
@@ -1401,21 +1402,9 @@ void KateMainWindow::activateWidget(QWidget *widget)
     }
 }
 
-void KateMainWindow::showDiff(const QByteArray &wordDiff, const DiffParams &params)
+void KateMainWindow::showDiff(const QByteArray &diff, const DiffParams &params)
 {
-    auto w = new DiffWidget(params, this);
-    if (!params.tabTitle.isEmpty()) {
-        w->setWindowTitle(params.tabTitle);
-    } else {
-        if (params.destFile.isEmpty())
-            w->setWindowTitle(i18n("Diff %1", Utils::fileNameFromPath(params.srcFile)));
-        else
-            w->setWindowTitle(i18n("Diff %1..%2", Utils::fileNameFromPath(params.srcFile), Utils::fileNameFromPath(params.destFile)));
-    }
-
-    w->setWindowIcon(QIcon::fromTheme(QStringLiteral("text-x-patch")));
-    addWidget(w);
-    w->openDiff(wordDiff);
+    DiffWidgetManager::openDiff(diff, params, this);
 }
 
 void KateMainWindow::showMessage(const QVariantMap &map)
