@@ -6,8 +6,7 @@
    SPDX-License-Identifier: LGPL-2.0-only
 */
 
-#ifndef __KATE_MAINWINDOW_H__
-#define __KATE_MAINWINDOW_H__
+#pragma once
 
 #include "diffparams.h"
 #include "katemdi.h"
@@ -61,8 +60,9 @@ public:
      * Construct the window and restore its state from given config if any
      * @param sconfig session config for this window, 0 if none
      * @param sgroup session config group to use
+     * @param userTriggered is the user creating this window or is it part of session restore / startup
      */
-    KateMainWindow(KConfig *sconfig, const QString &sgroup);
+    KateMainWindow(KConfig *sconfig, const QString &sgroup, bool userTriggered);
 
     /**
      * Destruct the nice window
@@ -155,7 +155,7 @@ private:
      */
     void setupImportantActions();
 
-    void setupMainWindow();
+    void setupMainWindow(KConfig *sconfig);
     void setupActions();
     bool queryClose() override;
 
@@ -186,13 +186,18 @@ public Q_SLOTS:
      */
     QSize sizeHint() const override;
 
+    /**
+     * Create a new window.
+     * Inherit size from current one.
+     * @return new window
+     */
+    KateMainWindow *newWindow() const;
+
     /**
      * slots used for actions in the menus/toolbars
      * or internal signal connections
      */
 private Q_SLOTS:
-    static void newWindow();
-
     void slotConfigure();
 
     void slotOpenWithMenuAction(QAction *a);
@@ -469,6 +474,11 @@ public Q_SLOTS:
         hideBottomViewBarForView(view);
     }
 
+    void insertWidgetInStatusBar(QWidget *w)
+    {
+        insertWidgetBeforeStatusbar(w);
+    }
+
     /**
      * Create a new toolview with unique \p identifier at side \p pos
      * with \p icon and caption \p text. Use the returned widget to embed
@@ -682,6 +692,18 @@ private:
      */
     KateOutputView *m_outputView = nullptr;
 
+    /**
+     * Diagnostics view at the bottom
+     */
+    QWidget *m_toolViewDiags = nullptr;
+    class DiagnosticsView *m_diagView = nullptr;
+
+public:
+    class DiagnosticsView *diagnosticsView()
+    {
+        return m_diagView;
+    }
+
 public:
     /**
      * The available actions for the mouse back button, used as indexing
@@ -734,5 +756,3 @@ protected:
     bool event(QEvent *e) override;
     void mousePressEvent(QMouseEvent *e) override;
 };
-
-#endif
